"""
This backend uses the Python 2D plotting library Matplotlib (available from
http://matplotlib.sourceforge.net). To use this backend, one can run a
script somefile.py like

  python somefile.py --SCITOOLS_easyviz_backend matplotlib

or one can specify the backend in the SciTools configuration file
scitools.cfg under the [easyviz] section

  [easyviz]
  backend = matplotlib

and then

  from scitools.std import *

or if just easyviz is needed

  from scitools.easyviz import *

REQUIREMENTS:

Matplotlib

"""
from __future__ import division

from common import *
from scitools.numpyutils import floor, linspace, array
from scitools.globaldata import DEBUG, VERBOSE
from scitools.misc import test_if_module_exists
from misc import _update_from_config_file

test_if_module_exists('matplotlib', msg='You need to install the Matplotlib package.', abort=False)

import matplotlib
import matplotlib.colors
# Override values from the matplotlib configuration file with values
# from scitools.cfg before importing pylab
_update_from_config_file(matplotlib.rcParams, section='matplotlib')
matplotlib.interactive(True)
from matplotlib.font_manager import fontManager, FontProperties
#import pylab
import matplotlib.pyplot as pylab
import re


class MatplotlibBackend(BaseClass):
    def __init__(self):
        BaseClass.__init__(self)
        self._init()

    def _init(self, *args, **kwargs):
        """Perform initialization that is special for this backend."""

        self.figure(self.getp('curfig'))

        # conversion tables for format strings:
        self._markers = {
            '': '',   # no marker
            '.': '.', # dot
            'o': 'o', # circle
            'x': 'x', # cross
            '+': '+', # plus sign
            '*': '+', # asterisk --> plus
            's': 's', # square,
            'd': 'd', # diamond,
            'v': 'v', # triangle (down),
            '^': '^', # triangle (up),
            '<': '<', # triangle (left),
            '>': '>', # triangle (right),
            'p': 'p', # pentagram,
            'h': 'h', # hexagram,
            }

        # convert table for colorbar location:
        self._colorbar_locations = {
            'North': None,
            'South': None,
            'East': None,
            'West': None,
            'NorthOutside': None,
            'SouthOutside': None,
            'EastOutside': None,
            'WestOutside': None,
            }

        if DEBUG:
            print "Setting backend standard variables"
            for disp in 'self._markers'.split():
                print disp, eval(disp)

    def _set_scale(self, ax):
        """Set linear or logarithmic (base 10) axis scale."""
        if DEBUG:
            print "Setting scales"
        scale = ax.getp('scale')
        if scale == 'loglog':
            # use logarithmic scale on both x- and y-axis
            self._g.gca().set_xscale('log')
            self._g.gca().set_yscale('log')
        elif scale == 'logx':
            # use logarithmic scale on x-axis and linear scale on y-axis
            self._g.gca().set_xscale('log')
            self._g.gca().set_yscale('linear')
        elif scale == 'logy':
            # use linear scale on x-axis and logarithmic scale on y-axis
            self._g.gca().set_xscale('linear')
            self._g.gca().set_yscale('log')
        elif scale == 'linear':
            # use linear scale on both x- and y-axis
            #self._g.gca().set_xscale('linear')
            #self._g.gca().set_yscale('linear')
            pass

    def _set_labels(self, ax):
        """Add text labels for x-, y-, and z-axis."""
        if DEBUG:
            print "Setting labels"
        xlabel = ax.getp('xlabel')
        ylabel = ax.getp('ylabel')
        zlabel = ax.getp('zlabel')
        xlabel = self._fix_latex(xlabel)
        ylabel = self._fix_latex(ylabel)
        zlabel = self._fix_latex(zlabel)
        if xlabel:
            # add a text label on x-axis
            self._g.xlabel(xlabel)
        if ylabel:
            # add a text label on y-axis
            self._g.ylabel(ylabel, rotation='vertical')
        if zlabel:
            # add a text label on z-axis
            pass

    def _set_title(self, ax):
        """Add a title at the top of the axis."""
        if DEBUG:
            print "Setting title"
        title = ax.getp('title')
        # The title can be a mix of math and text, leave this
        # to the user
        #title = self._fix_latex(title)
        if title:
            self._g.title(title)

    def _set_limits(self, ax):
        """Set axis limits in x, y, and z direction."""
        if DEBUG:
            print "Setting axis limits"
        mode = ax.getp('mode')
        if mode == 'auto':
            # let plotting package set 'nice' axis limits in the x, y,
            # and z direction. If this is not automated in the plotting
            # package, one can use the following limits:
            #xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            #self._g.axis('auto')
            pass
        elif mode == 'manual':
            # (some) axis limits are frozen
            xmin = ax.getp('xmin')
            xmax = ax.getp('xmax')
            if xmin is not None and xmax is not None:
                # set x-axis limits
                self._g.gca().set_xlim(xmin, xmax)
            else:
                # let plotting package set x-axis limits or use
                #xmin, xmax = ax.getp('xlim')
                pass

            ymin = ax.getp('ymin')
            ymax = ax.getp('ymax')
            if ymin is not None and ymax is not None:
                # set y-axis limits
                self._g.gca().set_ylim(ymin, ymax)
            else:
                # let plotting package set y-axis limits or use
                #ymin, ymax = ax.getp('ylim')
                pass

            zmin = ax.getp('zmin')
            zmax = ax.getp('zmax')
            if zmin is not None and zmax is not None:
                # set z-axis limits
                pass
            else:
                # let plotting package set z-axis limits or use
                #zmin, zmax = ax.getp('zlim')
                pass
        elif mode == 'tight':
            # set the limits on the axis to the range of the data. If
            # this is not automated in the plotting package, one can
            # use the following limits:
            #xmin, xmax, ymin, ymax, zmin, zmax = ax.get_limits()
            self._g.axis('tight')
        elif mode == 'fill':
            # not sure about this
            pass

    def _set_position(self, ax):
        """Set axes position."""
        rect = ax.getp('viewport')
        if rect:
            # axes position is defined. In Matlab rect is defined as
            # [left,bottom,width,height], where the four parameters are
            # location values between 0 and 1 ((0,0) is the lower-left
            # corner and (1,1) is the upper-right corner).
            # NOTE: This can be different in the plotting package.
            pass

    def _set_daspect(self, ax):
        """Set data aspect ratio."""
        if ax.getp('daspectmode') == 'manual':
            dar = ax.getp('daspect')  # dar is a list (len(dar) is 3).
            xmin = ax.getp('xmin')
            xmax = ax.getp('xmax')
            ymin = ax.getp('ymin')
            ymax = ax.getp('ymax')
            r = float(xmax-xmin)/(ymax-ymin)
            self._g.gca().set_aspect(r*dar[0])
        elif ax.getp('daspectmode') == 'equal':
            self._g.gca().set_aspect('equal')
        else:
            # daspectmode is 'auto'. Plotting package handles data
            # aspect ratio automatically.
            self._g.gca().set_aspect('auto')

    def _set_axis_method(self, ax):
        method = ax.getp('method')
        if method == 'equal':
            # tick mark increments on the x-, y-, and z-axis should
            # be equal in size.
            self._g.axis('equal')
        elif method == 'image':
            # same effect as axis('equal') and axis('tight')
            self._g.axis('image')
        elif method == 'square':
            # make the axis box square in size
            pass
        elif method == 'normal':
            # full size axis box
            pass
        elif method == 'vis3d':
            # freeze data aspect ratio when rotating 3D objects
            pass

    def _set_coordinate_system(self, ax):
        """
        Use either the default Cartesian coordinate system or a
        matrix coordinate system.
        """

        direction = ax.getp('direction')
        if direction == 'ij':
            # Use matrix coordinates. The origin of the coordinate
            # system is the upper-left corner. The i-axis should be
            # vertical and numbered from top to bottom, while the j-axis
            # should be horizontal and numbered from left to right.
            pass
        elif direction == 'xy':
            # use the default Cartesian axes form. The origin is at the
            # lower-left corner. The x-axis is vertical and numbered
            # from left to right, while the y-axis is vertical and
            # numbered from bottom to top.
            pass

    def _set_box(self, ax):
        """Turn box around axes boundary on or off."""
        if DEBUG:
            print "Setting box"
        if ax.getp('box'):
            # display box
            self._g.box(on=True)
        else:
            # do not display box
            self._g.box(on=False)

    def _set_grid(self, ax):
        """Turn grid lines on or off."""
        if DEBUG:
            print "Setting grid"
        if self._mpl3D:
            # Do not call self._g.grid for 3D surf plots
                return

        if ax.getp('grid'):
            # turn grid lines on
            self._g.grid(b=True)
        else:
            # turn grid lines off
            self._g.grid(b=False)

    def _set_hidden_line_removal(self, ax):
        """Turn on/off hidden line removal for meshes."""
        if DEBUG:
            print "Setting hidden line removal"
        if ax.getp('hidden'):
            # turn hidden line removal on
            pass
        else:
            # turn hidden line removal off
            pass

    def _set_colorbar(self, ax):
        """Add a colorbar to the axis."""
        if DEBUG:
            print "Setting colorbar"
        if self._mpl3D:
            # Do not call self._g.grid for 3D surf plots
                return

        cbar = ax.getp('colorbar')
        if cbar.getp('visible'):
            # turn on colorbar
            cbar_title = cbar.getp('cbtitle')
            cbar_location = self._colorbar_locations[cbar.getp('cblocation')]
            cbar2 = self._g.colorbar(orientation='vertical')
            if cbar_title:
                cbar2.set_label(cbar_title)
            #pos = [0.78, 0.1, 0.12, 0.8]
            #cbar2.ax.set_position(pos)
            #print cbar2.ax.get_position()
        else:
            # turn off colorbar
            pass

    def _set_caxis(self, ax):
        """Set the color axis scale."""
        if DEBUG:
            print "Setting caxis"
        if ax.getp('caxismode') == 'manual':
            cmin, cmax = ax.getp('caxis')
            # NOTE: cmin and cmax might be None:
            if cmin is None or cmax is None:
                cmin, cmax = [0,1]
            # set color axis scaling according to cmin and cmax
            if self._mplsurf is not None:
                pass # cannot handle 3D surf
            else:
                self._g.clim(cmin,cmax)
        else:
            # use autoranging for color axis scale
            pass

    def _set_colormap(self, ax):
        """Set the colormap."""
        if DEBUG:
            print "Setting colormap"
        cmap = ax.getp('colormap')
        # cmap is plotting package dependent
        # the colormap is set in

    def _set_view(self, ax):
        """Set viewpoint specification."""
        if DEBUG:
            print "Setting view"
        cam = ax.getp('camera')
        view = cam.getp('view')
        if view == 2:
            # setup a default 2D view
            pass
        elif view == 3:
            az = cam.getp('azimuth')
            el = cam.getp('elevation')
            if az is None or el is None:
                # azimuth or elevation is not given. Set up a default
                # 3D view (az=-37.5 and el=30 is the default 3D view in
                # Matlab).
                pass
            else:
                # set a 3D view according to az and el
                pass

            if cam.getp('cammode') == 'manual':
                # for advanced camera handling:
                roll = cam.getp('camroll')
                zoom = cam.getp('camzoom')
                dolly = cam.getp('camdolly')
                target = cam.getp('camtarget')
                position = cam.getp('campos')
                up_vector = cam.getp('camup')
                view_angle = cam.getp('camva')
                projection = cam.getp('camproj')

    def _set_axis_props(self, ax):
        if DEBUG:
            print "Setting axis properties"
        self._set_title(ax)
        self._set_scale(ax)
        self._set_axis_method(ax)
        self._set_limits(ax)
        self._set_position(ax)
        self._set_daspect(ax)
        self._set_coordinate_system(ax)
        self._set_hidden_line_removal(ax)
        self._set_colorbar(ax)
        self._set_caxis(ax)
        self._set_colormap(ax)
        self._set_view(ax)
        if ax.getp('visible'):
            self._set_labels(ax)
            self._set_box(ax)
            self._set_grid(ax)
        else:
            # turn off all axis labeling, tickmarks, and background
            self._g.axis('off')

    def _get_linespecs(self, item):
        """
        Return the line marker, line color, line style, and
        line width of the item.
        """
        marker = self._markers[item.getp('linemarker')]
        color = item.getp('linecolor')
        style = item.getp('linetype')
        width = item.getp('linewidth')
        if width:
            width = float(width)
        return marker, color, style, width

    def _fix_latex(self, legend):
        """Enclose legend in $$ if latex syntax is detected."""
        # We always support latex syntax either through direct
        # use of latex (text.usetex=true) or through the native mathtext.
        #if not matplotlib.rcParams['text.usetex']:
        #    return legend

        legend = legend.strip()
        if len(legend) >= 2 and legend[0] != '$' and legend[-1] != '$':
            # else: assume correct latex syntax, otherwise fix
            #print '....fix', legend,
            if '**' in legend:
                legend = legend.replace('**', '^')
            if '*' in legend:
                #legend = legend.replace('*', '\\cdot')
                legend = legend.replace('*', ' ')

            chars = '\\', '^', '_'
            latex = False
            for c in chars:
                if c in legend:
                    latex = True
                    break
            if latex:
                # make this math:
                legend = '$' + legend + '$'

                # enclose words in \hbox{}'es:
                # (not successful enough)
                #word = r'(\s?[^\\][A-Za-z][a-z]*[ :;,$)] ?)'
                #word = r'(\s?[A-Za-z][a-z]*[ :;,$)])'
                #word = r'(\w[ :;,$)])'
                #word = r'(\b[A-Za-z][a-z]*[ :;,$)]\b)'
                #if re.search(word, legend):
                #    legend = re.sub(word, r'\hbox{\g<1>}', legend)
                # remove internal $ chars:
                legend = legend.replace('$', '')
                legend = '$' + legend + '$'

                # fix sin, cos, exp, ln, log, etc:
                def _fix_func(func, newfunc, legend):
                    if re.search(r'[^\\]'+func, legend):
                        legend = legend.replace(func, '\\'+newfunc)
                    return legend
                funcs = 'sin', 'cos', 'tan', 'exp', 'ln', 'log', \
                        'sinh', 'cosh', 'tanh'
                for func in funcs:
                    legend = _fix_func(func, func, legend)
                funcs = [('atan', 'arctan'), ('asin', 'arcsin'),
                         ('acos', 'arccos'),]
                for func, newfunc in funcs:
                    legend = _fix_func(func, newfunc, legend)
                #print 'after fix:', legend
        return legend

    def _add_line(self, item):
        """Add a 2D or 3D curve to the scene."""
        if DEBUG:
            print "Adding a line"
        # get data:
        x = squeeze(item.getp('xdata'))
        y = squeeze(item.getp('ydata'))
        z = item.getp('zdata')
        if z is not None:
            z = squeeze(z)
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)

        fmt = marker+color+style
        if not width:
            width = 1.0

        if z is not None:
            # zdata is given, add a 3D curve:
            print "No support for plot3 in Matplotlib."
        else:
            # no zdata, add a 2D curve:
            #l, = self._g.plot(x,y,fmt,linewidth=width)
            l = self._g.plot(x,y,fmt,linewidth=width)
            legend = item.getp('legend')
            legend = self._fix_latex(legend)
            if legend:
                l[0].set_label(legend)

    def _add_filled_line(self, item):
        """Add a 2D or 3D filled curve."""
        if DEBUG:
            print "Adding a line"
        # get data:
        x = squeeze(item.getp('xdata'))
        y = squeeze(item.getp('ydata'))
        z = item.getp('zdata')
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)

        if not width:
            width = 1.0

        facecolor = item.getp('facecolor')
        if not facecolor:
            if not color:
                color = 'b'
            facecolor = color
        edgecolor = item.getp('edgecolor')
        if not edgecolor:
            edgecolor = 'k'  # use black for now
            # FIXME: edgecolor should be ax.getp('fgcolor') by default
        opacity = item.getp('material').getp('opacity')
        if opacity is None:
            opacity = 1.0

        if z is not None:
            # zdata is given, add a filled 3D curve:
            print "No support for fill3 in Matplotlib."
        else:
            # no zdata, add a filled 2D curve:
            l = self._g.fill(x, y, fc=facecolor, ec=edgecolor,
                              linewidth=width, alpha=opacity)
            legend = item.getp('legend')
            legend = self._fix_latex(legend)
            if legend:
                l[0].set_label(legend)

    def _add_bar_graph(self, item, shading='faceted'):
        if DEBUG:
            print "Adding a bar graph"
        # get data:
        x = squeeze(item.getp('xdata'))
        y = squeeze(item.getp('ydata'))
        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)

        edgecolor = item.getp('edgecolor')
        if not edgecolor:
            edgecolor = 'k'  # use black for now
            # FIXME: edgecolor should be same as ax.getp('fgcolor') by default
        facecolor = item.getp('facecolor')
        if not facecolor:
            facecolor = color
        opacity = item.getp('material').getp('opacity')
        if opacity is None:
            opacity = 1.0

        if rank(y) == 1:
            y = reshape(y,(len(y),1))
        nx, ny = shape(y)

        step = item.getp('barstepsize')/10

        center = floor(ny/2)
        start = -step*center
        stop = step*center
        if not ny%2:
            start += step/2
            stop -= step/2
        a = linspace(start,stop,ny)

        barwidth = item.getp('barwidth')/10

        hold_state = self._g.ishold()
        self._g.hold(True)
        colors = PlotProperties._colors + matplotlib.colors.cnames.values()
        for j in range(ny):
            y_ = y[:,j]
            x_ = array(range(nx)) + a[j] - barwidth/2
            if not facecolor:
                c = colors[j]
            else:
                c = facecolor
            self._g.bar(x_, y_, width=barwidth, color=c,
                        ec=edgecolor, alpha=opacity)
        self._g.hold(hold_state)

        barticks = item.getp('barticks')
        if barticks is None:
            barticks = x
        if item.getp('rotated_barticks'):
            self._g.xticks(range(len(x)), barticks, rotation=90)
        else:
            self._g.xticks(range(len(x)), barticks)

    def _add_surface(self, item, shading='faceted', colormap=None,
                     showcolorbar=False, zmin=None, zmax=None):
        if DEBUG:
            print "Adding a surface"
        x = squeeze(item.getp('xdata'))  # grid component in x-direction
        y = squeeze(item.getp('ydata'))  # grid component in y-direction
        z = item.getp('zdata')           # scalar field
        c = item.getp('cdata')           # pseudocolor data (can be None)
        legend = item.getp('legend')
        legend = self._fix_latex(legend)

        if colormap is None or colormap == 'default':
            colormap = self._g.cm.get_cmap('jet')

        if shape(x) != shape(z) or shape(y) != shape(z):
            x, y = meshgrid(x, y, sparse=False,
                            indexing=item.getp('indexing'))

        opacity = item.getp('material').getp('opacity')
        if opacity is None:
            opacity = 1.0

        contours = item.getp('contours')
        if contours:
            # the current item is produced by meshc or surfc and we
            # should therefore add contours at the bottom:
            #self._add_contours(contours, placement='bottom')
            print "No support for meshc/surfc in Matplotlib."

        if item.getp('wireframe'):
            # wireframe mesh (as produced by mesh or meshc)
            fig = self._g.gcf()
            ax = fig.gca(projection='3d')
            h = ax.plot_wireframe(x, y, z)
            if legend:
                h.set_label(legend)
        else:
            # colored surface (as produced by surf, surfc, or pcolor)
            # use keyword argument shading to set the color shading mode
            function = item.getp('function')
            if function == 'pcolor':
                h = self._g.pcolor(x, y, z, shading=shading,
                                   cmap=colormap, alpha=opacity)
            else:
                # This is really a hack to use 3D surfaces in matplotlib...
                from mpl_toolkits.mplot3d import Axes3D
                fig = self._g.gcf()
                ax = fig.gca(projection='3d')
                if self._mplsurf is not None:
                    ax.collections.remove(self._mplsurf)
                self._mplsurf = ax.plot_surface(x, y, z, rstride=1, cstride=1,
                                                linewidth=0, cmap=colormap,
                                                antialiased=False)
                if showcolorbar and not self._mpl3D:
                    # Show colorbar only for the first plot in an animation
                    fig.colorbar(self._mplsurf, shrink=1.0, aspect=15,
                                 orientation='vertical',)
                if zmin is not None and zmax is not None:
                    ax.set_zlim3d(zmin, zmax)
                self._mpl3D = True
                # Problem: cannot fix color scale (or?)

            if legend:
                h.set_label(legend)

    def _add_contours(self, item, placement=None, colormap=None):
        # The placement keyword can be either None or 'bottom'. The
        # latter specifies that the contours should be placed at the
        # bottom (as in meshc or surfc).
        if DEBUG:
            print "Adding contours"
        x = squeeze(item.getp('xdata'))  # grid component in x-direction
        y = squeeze(item.getp('ydata'))  # grid component in y-direction
        z = item.getp('zdata')           # scalar field
        legend = item.getp('legend')
        legend = self._fix_latex(legend)

        if colormap is None or colormap == 'default':
            colormap = self._g.cm.get_cmap('jet')

        if shape(x) != shape(z) or shape(y) != shape(z):
            x, y = meshgrid(x, y, sparse=False,
                            indexing=item.getp('indexing'))

        opacity = item.getp('material').getp('opacity')
        if opacity is None:
            opacity = 1.0

        filled = item.getp('filled')  # draw filled contour plot if True

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass
        else:
            clevels = cvector

        contour_cmd = self._g.contour
        location = item.getp('clocation')
        if location == 'surface':
            # place the contours at the corresponding z level (contour3)
            print "No support for contour3 in Matplotlib."
        elif location == 'base':
            if placement == 'bottom':
                # place the contours at the bottom (as in meshc or surfc)
                pass
            else:
                # standard contour plot
                pass
        if filled:
            contour_cmd = self._g.contourf
        kwargs = {'cmap': colormap, 'alpha': opacity}

        # get line specifiactions:
        marker, color, style, width = self._get_linespecs(item)
        if color:
            kwargs['colors'] = color
            kwargs['cmap'] = None

        if width:
            kwargs['linewidths'] = width

        if legend:
            kwargs['label'] = legend
        cs = contour_cmd(x,y,z,clevels,**kwargs)

        if item.getp('clabels'):
            # add labels on the contour curves
            self._g.clabel(cs)

    def _add_vectors(self, item):
        if DEBUG:
            print "Adding vectors"
        # uncomment the following command if there is no support for
        # automatic scaling of vectors in the current plotting package:
        #item.scale_vectors()

        # grid components:
        x = squeeze(item.getp('xdata'))
        y = squeeze(item.getp('ydata'))
        z = item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # get line specifiactions (marker='.' means no marker):
        marker, color, style, width = self._get_linespecs(item)
        indexing = item.getp('indexing')

        legend = item.getp('legend')
        legend = self._fix_latex(legend)

        # scale the vectors according to this variable (scale=0 should
        # turn off automatic scaling):
        scale = item.getp('arrowscale')
        if scale == 0:
            scale = None

        filled = item.getp('filledarrows') # draw filled arrows if True

        if z is not None and w is not None:
            # draw velocity vectors as arrows with components (u,v,w) at
            # points (x,y,z):
            print "No support for quiver3 in Matplotlib."
        else:
            # draw velocity vectors as arrows with components (u,v) at
            # points (x,y):
            if shape(x) != shape(u) and shape(y) != shape(u):
                x, y = meshgrid(x, y, sparse=False, indexing=indexing)
            if not color:
                c = u**2+v**2  # color arrows by magnitude
                h = self._g.quiver(x,y,u,v,c,scale=scale)
            else:
                h = self._g.quiver(x,y,u,v,scale=scale,color=color)
            if legend:
                h.set_label(legend)

    def _add_streams(self, item):
        if DEBUG:
            print "Adding streams"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        # vector components:
        u, v, w = item.getp('udata'), item.getp('vdata'), item.getp('wdata')
        # starting positions for streams:
        sx, sy, sz = item.getp('startx'), item.getp('starty'), item.getp('startz')

        if item.getp('tubes'):
            # draw stream tubes from vector data (u,v,w) at points (x,y,z)
            n = item.getp('n') # no points along the circumference of the tube
            scale = item.getp('tubescale')
            pass
        elif item.getp('ribbons'):
            # draw stream ribbons from vector data (u,v,w) at points (x,y,z)
            width = item.getp('ribbonwidth')
            pass
        else:
            if z is not None and w is not None:
                # draw stream lines from vector data (u,v,w) at points (x,y,z)
                pass
            else:
                # draw stream lines from vector data (u,v) at points (x,y)
                pass
            pass

    def _add_isosurface(self, item):
        if DEBUG:
            print "Adding a isosurface"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume
        c = item.getp('cdata')  # pseudocolor data
        isovalue = item.getp('isovalue')

    def _add_slices(self, item):
        if DEBUG:
            print "Adding slices in a volume"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass
        pass

    def _add_contourslices(self, item):
        if DEBUG:
            print "Adding contours in slice planes"
        # grid components:
        x, y, z = item.getp('xdata'), item.getp('ydata'), item.getp('zdata')
        v = item.getp('vdata')  # volume

        sx, sy, sz = item.getp('slices')
        if rank(sz) == 2:
            # sx, sy, and sz defines a surface
            pass
        else:
            # sx, sy, and sz is either numbers or vectors with numbers
            pass

        cvector = item.getp('cvector')
        clevels = item.getp('clevels')  # number of contour levels per plane
        if cvector is None:
            # the contour levels are chosen automatically
            #cvector =
            pass
        pass

    def _set_figure_size(self, fig):
        if DEBUG:
            print "Setting figure size"
        width, height = fig.getp('size')
        if width and height:
            # set figure width and height
            fig = self._g.gcf()
            fig.set_size_inches(width,height)
        else:
            # use the default width and height in plotting package
            pass

    def figure(self, *args, **kwargs):
        # Extension of BaseClass.figure:
        # add a plotting package figure instance as fig._g and create a
        # link to it as self._g
        fignum = BaseClass.figure(self, *args, **kwargs)
        fig = self.gcf()
        try:
            fig._g
        except:
            # create plotting package figure and save figure instance
            # as fig._g
            if DEBUG:
                name = 'Fig ' + str(fignum)
                print "creating figure %s in backend" % name

            fig._g = pylab

        self._g = fig._g # link for faster access
        self._mpl3D = False
        self._mplsurf = None
        self._texts = {}  # store calls to text (for replot)
        return fignum

    def _replot(self):
        """Replot all axes and all plotitems in the backend."""
        # NOTE: only the current figure (gcf) is redrawn.
        if DEBUG:
            print "Doing replot in backend"

        # turn off interactive in pyplot temporarily:
        old_pylab_interactive_state = self._g.isinteractive()
        self._g.ioff()

        fig = self.gcf()
        try:
            fig._g
        except:
            self.figure(self.getp('curfig'))
        self._g.figure(self.getp('curfig'))

        # reset the plotting package instance in fig._g now if needed
        if not self._mpl3D:
            self._g.clf()

        self._set_figure_size(fig)

        nrows, ncolumns = fig.getp('axshape')
        for axnr, ax in fig.getp('axes').items():
            if ax.getp('numberofitems') == 0:
                continue
            if nrows != 1 or ncolumns != 1:
                # create axes in tiled position
                # this is subplot(nrows, ncolumns, axnr)
                self._g.subplot(nrows, ncolumns, axnr)
            else:
                rect = ax.getp('viewport')
                if isinstance(rect, (list,tuple)) and len(rect) == 4 and \
                       ax.getp('pth') is None:
                    self._g.axes(rect)
            legends = False
            plotitems = ax.getp('plotitems')
            plotitems.sort(self._cmpPlotProperties)
            for item in plotitems:
                func = item.getp('function') # function that produced this item
                if isinstance(item, Line):
                    if func == 'fill':
                        self._add_filled_line(item)
                    else:
                        self._add_line(item)
                elif isinstance(item, Bars):
                    self._add_bar_graph(item, shading=ax.getp('shading'))
                elif isinstance(item, Surface):
                    self._add_surface(item,
                      shading=ax.getp('shading'),
                      colormap=ax.getp('colormap'),
                      showcolorbar=ax.getp('colorbar').getp('visible'),
                      zmin=ax.getp('zmin'),
                      zmax=ax.getp('zmax'))
                elif isinstance(item, Contours):
                    self._add_contours(item, colormap=ax.getp('colormap'))
                elif isinstance(item, VelocityVectors):
                    self._add_vectors(item)
                elif isinstance(item, Streams):
                    self._add_streams(item)
                elif isinstance(item, Volume):
                    if func == 'isosurface':
                        self._add_isosurface(item)
                    elif func == 'slice_':
                        self._add_slices(item)
                    elif func == 'contourslice':
                        self._add_contourslices(item)
                legend = item.getp('legend')
                legend = self._fix_latex(legend)
                if legend:
                    # add legend to plot
                    legends = True
                if ax.getp('numberofitems') > 1 and not self._g.ishold():
                    self._g.hold(True)

            if legends:
                try:
                    loc = ax.getp('legend_loc')
                except KeyError:
                    loc = 'best'
                try:
                    fancybox = ax.getp('legend_fancybox')
                except KeyError:
                    fancybox = True
                self._g.legend(loc=loc, fancybox=fancybox)

            self._set_axis_props(ax)

        # Display texts
        for args in self._texts:
            self.text(args[0], args[1], args[2],
                      fontname=args[3], fontsize=args[4])

        # set back the interactive state in pylab:
        if old_pylab_interactive_state:
            self._g.ion()

        self._g.draw()
        if self.getp('show'):
            # display plot on the screen
            if DEBUG:
                print "\nDumping plot data to screen\n"
                debug(self)
            self._g.figure(self.getp('curfig'))  # raise figure
            # Or is there a better way to draw the current figure without
            # calling pylab.show()?
            #self._g.show()

    def text(self, x, y, text,
             fontname=Axis._local_prop['fontname'],
             fontsize=Axis._local_prop['fontsize']):
        """Write text at position (x,y) in a curveplot."""
        self._g.text(x, y, text, family=fontname, size=fontsize)
        self._texts[(x, y, text, fontname, fontsize)] = None


    def hardcopy(self, filename, **kwargs):
        """
        Supported extensions: .eps, .jpg, .pdf, .png, .ps, and .svg
        This is dependent on the Matplotlib backend.

        Optional arguments:

          dpi         -- image resolution. Default is 150.
          orientation -- 'portrait' (default) or 'landscape'. Only available
                         for PostScript output.

        """
        self.setp(**kwargs)
        color = self.getp('color')
        replot = kwargs.get('replot', True)
        if replot:
            self._replot()

        if DEBUG:
            print "Hardcopy to %s" % filename

        dpi = kwargs.get('dpi', 150)
        orientation = kwargs.get('orientation', 'portrait')

        self._g.savefig(filename,
                        facecolor='w',
                        edgecolor='w',
                        orientation=orientation)

    def clf(self):
        self._g.clf()
        BaseClass.clf(self)

    def closefig(self, arg=None):
        if arg is None:
            num = self.getp('curfig')  # close current figure
        elif arg in self._figs.keys():
            num = arg
        elif arg in self._figs.values():
            for fignr, fig in self._figs.items():
                if fig == arg:
                    num = fignr
                    break
        else:
            raise ValueError("closefig: cannot close figure '%s'" % arg)
        self._g.close(num)
        #del self._figs[num]._g
        #del self._figs[num]

    def closefigs(self):
        for key in self._figs.keys():
            self.closefig(key)
        del self._g
        BaseClass.closefigs(self)

    # Colormap methods:
    def hsv(self, m=None):
        return pylab.cm.get_cmap('hsv', m)

    def hot(self, m=None):
        return pylab.cm.get_cmap('hot', m)

    def gray(self, m=None):
        return pylab.cm.get_cmap('gray', m)

    def bone(self, m=None):
        return pylab.cm.get_cmap('bone', m)

    def copper(self, m=None):
        return pylab.cm.get_cmap('copper', m)

    def pink(self, m=None):
        return pylab.cm.get_cmap('pink', m)

    def white(self, m=None):
        raise NotImplementedError('white not implemented in class %s' % \
                                  self.__class__.__name__)

    def flag(self, m=None):
        return pylab.cm.get_cmap('flag', m)

    def lines(self, m=None):
        raise NotImplementedError('lines not implemented in class %s' % \
                                  self.__class__.__name__)

    def colorcube(self, m=None):
        raise NotImplementedError('colorcube not implemented in class %s' % \
                                  self.__class__.__name__)

    def vga(self, m=None):
        raise NotImplementedError('vga not implemented in class %s' % \
                                  self.__class__.__name__)

    def jet(self, m=None):
        return pylab.cm.get_cmap('jet', m)

    def prism(self, m=None):
        return pylab.cm.get_cmap('prism', m)

    def cool(self, m=None):
        return pylab.cm.get_cmap('cool', m)

    def autumn(self, m=None):
        return pylab.cm.get_cmap('autumn', m)

    def spring(self, m=None):
        return pylab.cm.get_cmap('spring', m)

    def winter(self, m=None):
        return pylab.cm.get_cmap('winter', m)

    def summer(self, m=None):
        return pylab.cm.get_cmap('summer', m)


    # Now we add the doc string from the methods in BaseClass to the
    # methods that are reimplemented in this backend:
    for cmd in BaseClass._matlab_like_cmds:
        if not '__' in cmd and hasattr(BaseClass, cmd):
            m1 = eval('BaseClass.%s' % cmd)
            try:
                m2 = eval('%s' % cmd)
            except NameError:
                pass
            else:
                if m1.__doc__ != m2.__doc__:
                    if m2.__doc__ is None:
                        m2.__doc__ = ""
                    m2.__doc__ = m1.__doc__ + m2.__doc__


plt = MatplotlibBackend()  # create backend instance
use(plt, globals())        # export public namespace of plt to globals()

# We should close all figure windows on program exit:
import atexit
atexit.register(close, 'all')

backend = os.path.splitext(os.path.basename(__file__))[0][:-1]



#!/usr/bin/env python
"""Make a string with a mathematical expression behave as a Python function."""
from __future__ import division

# Default import of mathematical functions (in case the user
# supplies expressions with math functions and does not provide
# a globals keyword to the constructor with appropriate modules
# defining these math functions):
# from math import *
math_functions = ['acos', 'asin', 'atan', 'atan2', 'ceil', 'cos',
                  'cosh', 'exp', 'fabs', 'floor', 'log', 'log10',
                  'pi', 'pow', 'sin', 'sinh', 'sqrt', 'tan', 'tanh']
s = 'from math import ' + ', '.join(math_functions)
exec s
# Problem: vectorized expressions require NumPy versions
# of the math functions. We try to detect errors arising from
# such lacking imports.

# The first edition of the "Python for Computational Science" book
# introduced the classes StringFunction1x, StringFunction1, and
# StringFunction. These are now (for the second edition) named
# StringFunction_v3, StringFunction_v4, and StringFunction_v5,
# respectively.


# The following implementation of StringFunction is an
# improved version compared to the one explained in the
# first edition of the book "Python for Computational Science".
# The new version is created by Mario Pernici <Mario.Pernici@mi.infn.it>
# and Hans Petter Langtangen <hpl@simula.no>. The basic idea is to
# build a lambda function out of the string expression and define
# self.__call__ to be this lambda function.

import re

class StringFunction:
    """
    Representation of a string formula as a function of one or
    more variables, optionally with parameters.

    Example on usage:

    >>> from scitools.StringFunction import StringFunction
    >>> f = StringFunction('1+sin(2*x)')
    >>> f(1.2)
    1.6754631805511511

    >>> f = StringFunction('1+sin(2*t)', independent_variable='t')
    >>> f(1.2)
    1.6754631805511511

    >>> f = StringFunction('1+A*sin(w*t)', independent_variable='t', \
                           A=0.1, w=3.14159)
    >>> f(1.2)
    0.94122173238695939
    >>> f.set_parameters(A=1, w=1)
    >>> f(1.2)
    1.9320390859672263

    >>> f(1.2, A=2, w=1)   # can also set parameters in the call
    2.8640781719344526

    >>> # function of two variables:
    >>> f = StringFunction('1+sin(2*x)*cos(y)', \
                           independent_variables=('x','y'))
    >>> f(1.2,-1.1)
    1.3063874788637866

    >>> f = StringFunction('1+V*sin(w*x)*exp(-b*t)', \
                           independent_variables=('x','t'))
    >>> f.set_parameters(V=0.1, w=1, b=0.1)
    >>> f(1.0,0.1)
    1.0833098208613807
    >>> str(f)  # print formula with parameters substituted by values
    '1+0.1*sin(1*x)*exp(-0.1*t)'
    >>> repr(f)
    "StringFunction('1+V*sin(w*x)*exp(-b*t)', independent_variables=('x', 't'), b=0.10000000000000001, w=1, V=0.10000000000000001)"

    >>> # vector field of x and y:
    >>> f = StringFunction('[a+b*x,y]', \
                           independent_variables=('x','y'))
    >>> f.set_parameters(a=1, b=2)
    >>> f(2,1)  # [1+2*2, 1]
    [5, 1]

    StringFunction expressions may contain fractions like 1/2 and these
    always result in float division (not integer division). Here is
    an example:

    >>> from scitools.StringFunction import StringFunction
    >>> f = StringFunction('1/4 + 1/2*x')
    >>> f(2)
    1.25


    The string parameter can, instead of a valid Python expression,
    be a function in a file (module). The string is then the
    complete path to the function, typically of the form
    somepackage.somemodule.function_name. This functionality is useful
    when simple string formulas cannot describe the function, e.g., when
    there are multiple if-branches inside the function expression.

    As an example, there is a function called _test_function::

        def _test_function(x, c=0, a=1, b=2):
            if x > c:
                return a*(x-c) + b
            else:
                return -a*(x-c) + b

    in the module scitools.misc (i.e., the misc.py file in the scitools
    package). We can then specify the complete path of this function
    as "string expression":

    >>> f = StringFunction('scitools.misc._test_function', independent_variable='x', a=10)
    >>> f(4)  # 10*(4-0) + 2 = 42
    42

    (Note that in Python 2.5 the _test_function can be coded as a
    simple string expression a*(x-c)+b if x > c else -a*(x-c)+b.)

    Giving the name of a function in a file (module) is convenient in
    user interfaces because the user can then write the name of
    the function as a standard Python module.function path. StringFunction
    turns this name, as a string, into a working module.function path.

    Troubleshooting:

    1)
    The StringFunction class can work with sin, cos, exp, and other
    mathematical functions if the argument is a scalar (float or int) type.
    If the argument is a vector, the NumPy versions of sin, cos,
    exp, etc., are needed. A common error message in the latter case is::

       TypeError: only rank-0 arrays can be converted to Python scalars.

    Make something like::

       from numpy import *
       # or
       from scitools.std import *
       # or
       from numpy import sin, cos

    in the calling code and supply globals=globals() as argument to
    the constructor::

       f = StringFunction('1+x*y', independent_variables=('x', 'y'),
                          globals=globals())
       # f(p,q) will now work for NumPy arrays p and q.

    You can also omit the globals argument when constructing the
    StringFunction and later call
    f.vectorize(globals())
    to allow array arguments.

    2) StringFunction builds a lambda function and evaluates this.
    You can see the lambda function as a string by accessing the
    _lambda attribute.
    """
    def __init__(self, expression, **kwargs):
        self._f = str(expression)  # ensure a string

        # check if expression is a function in a module:
        self._function_in_module = None

        # a module function specification is on the form
        # [A-Za-z_][A-Za-z0-9_.]x where x+1 is the len(expression)
        # but there MUST be a dot in there
        pattern = r'[A-Za-z_][A-Za-z0-9_.]{%d}' % (len(expression)-1)
        if "." in expression:
            if re.search(pattern, expression):
                parts = expression.split('.')
                module = '.'.join(parts[:-1])
                function = parts[-1]
                self._function_in_module = (module, function)

        # self._var holds the independent variables in a tuple:
        if 'independent_variable' in kwargs:  # allow "variable" and "variables"
            # note that tuple(string) gives a tuple of the characters
            # so we need to be careful (if the indep. var has more than
            # one character)
            name = kwargs['independent_variable'] # 'x', 't', 'dudt' etc.
            if not isinstance(name, str):
                raise ValueError(
                    'name "%s" of independent variable is illegal' % name)
            self._var = (name,)
        else:
            names = kwargs.get('independent_variables', ('x',))
            if isinstance(names, str):
                names = (names,)
            elif not isinstance(names, (list,tuple)):
                raise ValueError(
                    'independent variables=%s is invalid' % names)
            self._var = tuple(names)

        # user's globals() array (with relevant imported modules/functions):
        if 'globals' in kwargs:
            if kwargs['globals'] is None:
                self._globals = globals()
            else:
                self._globals = kwargs['globals']
        else:
            self._globals = globals()

        self._prms = kwargs.copy()
	try:    del self._prms['independent_variable']
        except: pass
        try:    del self._prms['independent_variables']
        except: pass
        try:    del self._prms['globals']
        except: pass
        try:
            # may fail if not all parameters are defined yet
            self._build_lambda()
        except NameError, e:
            #print e
            pass # ok at this stage: parameters might be missing

    def _build_lambda(self):
        """
        Translate the expression to a lambda function taking the
        independent variables as positional arguments and the
        parameters as keyword arguments.
        The idea is due to Mario Pernici <Mario.Pernici@mi.infn.it>.
        """
        args = ', '.join(self._var)
        s = 'lambda ' + args

        # add parameters as keyword arguments:
        if self._prms:
            kwargs = ', '.join(['%s=%s' % (k, self._prms[k]) \
                                for k in self._prms])
            s += ', ' + kwargs
        else:
            kwargs = ''

        if self._function_in_module is None:
            # insert string expression as body in the lambda function:
            s += ': ' + self._f
        else:
            exec('import ' + self._function_in_module[0])
            # let lambda call a function in a file (module):
            s += ', module=%s: module.%s(%s, %s)' % \
                 (self._function_in_module[0],
                  self._function_in_module[1],
                  args, kwargs)
            # note: we could use self._f directly here (giving the
            # full module path), but then we need to do import first,
            # all this is done in the __init__ and then it is simpler
            # to just let self_function_in_module point to the imported
            # function

        self._lambda = s # store lambda function code; just for convenience

        try:
            if self._function_in_module is None:
                try:
                    self.__call__ = eval(s, self._globals)
                except Exception, e:
                    print """
Making StringFunction with formula %s failed!
Tried to build a lambda function:\n %s""" % (self._f, s)
                    raise e

                ## the following makes all instances have the same function :-(
                ##StringFunction.__call__ = eval(s, self._globals)
            else:
                # didn't work with self._globals...and we don't need it...????
                self.__call__ = eval(s, globals(), locals())
                #self.__call__ = eval(s)
                #print 'call is', self.__call__

        except NameError, e:
            prm = str(e).split()[1]
            raise NameError, 'name "%s" is not defined - if it is '\
                  'a parameter,\nset it in the constructor or the '\
                  'set_parameters method, or provide\nglobals=globals() '\
                  'in the constructor if "%s" is a global name in the '\
                  'calling code.' % (prm, prm)


    def set_parameters(self, **kwargs):
        """Set keyword parameters in the function."""
        self._prms.update(kwargs)
        self._build_lambda()

    def vectorize(self, globals_dict):
        """
        Allow the StringFunction object to take NumPy array
        arguments. The calling code must have done a
        from numpy import * or similar and send the globals()
        dictionary as the argument globals_dict.
        Alternatively, the globals() dictionary can be supplied
        as a globals keyword argument to the constructor.
        """
        self._globals = globals_dict
        self._build_lambda()

    def troubleshoot(self, *args, **kwargs):
        """
        Perform function evaluation call with lots of testing to
        try to help the user with problems.
        """
        try:
            v = self(*args, **kwargs)
        except TypeError, e:
            if str(e).find('only rank-0 arrays can be converted to Python scalars') != -1:

                print '\nThe call resulted in the exception TypeError:'
                print e

                # *args contains NumPy arrays and the operations in
                # the string formula are not compatible

                # do we have intrinsic math functions of wrong type?
                math_funcs = False; not_NumPy = False
                for f in math_functions:
                    if self._f.find(f) != -1:
                        math_funcs = True
                        if str(type(f))[7:-2] != 'ufunc':
                            not_NumPy = True
                        break
                if math_funcs and not_NumPy:
                    print '\nThis message is caused by using scalar math\n'\
                          'functions (like %s) with array arguments.\n'\
                          'Make some\nfrom numarray import *\n'\
                          'or similar in the calling code and '\
                          'supply the globals=globals() constructor\n'\
                          'argument when creating the StringFunction instance.'\
                          % f
                else:
                    print 'Internal error - this should not happen...'
            else:
                print e
        except NameError, e:
            print e
        else:
            print 'This call worked perfectly!'

    def __str__(self):
        """
        Return the string function formula as a string, with
        parameters substituted by their values.

        The return value can be used when creating Fortran or C/C++
        functions out of the string formula:
        f = StringFunction('a + p*x', a=1, p=0)
        somefile.write('double somefunc(double x) { return %s; }' % str(f))
        """
        s = self._f  # formula with parameter names and indep. variables

        # Substitute parameter names by their numerical values.
        # Start with the most complicated parameter names
        # (this algorithm may fail).
        prm_names = self._prms.keys()
        prm_names.sort(lambda a, b: cmp(len(a), len(b)))
        prm_names.reverse()
        for name in prm_names:
            s = s.replace(name, str(self._prms[name]))
        return s

    def __repr__(self):
        """Return the code required to reconstruct this instance."""
        kwargs = ', '.join(['%s=%s' % (key, repr(value)) \
                            for key, value in self._prms.items()])
        return """StringFunction(%s, independent_variables=%s, %s)""" % \
               (repr(self._f), repr(self._var), kwargs)

    # The next code generation functions work only for scalar
    # function values, not vector values (can extend to vectors by
    # checking if self._f has a list form, and then include the
    # return value as an array argument in the functions).

    def _no_of_vector_components(self):
        """
        Return the number of vector components in the string
        expression.
        """
        ni = len(self._var)
        if ni == 1:
            return 1
        # function of more than one variable may be a vector field:
        args = tuple([0]*ni)  # try (0,0,...) as indep. variables
        try:
            v = self(args)
            return ni
        except:
            # try another argument (in case (0,0,...) caused wrong calculations:
            args = tuple([1.105]*ni)
            v = self(args)
            return ni


    def Cpp_code(self, function_name='somefunc'):
        """
        Dump the string expression to C++.
        In C++ we use a plain function if there are no parameters,
        otherwise we use a function object with operator() for
        the function evaluation.
        """
        varlist = ', '.join(['double %s' % var for var in self._var])
        expr = self._f
        # ** does not work in C, instead of doing sophisticated
        # parsing and edit, provide an error message
        self._pow_check()
        if self._prms:
            decl = ' '.join(['double %s;' % name for name in self._prms])
            prms = ', '.join(['double %s_=%s' % (name, self._prms[name]) \
                    for name in self._prms])
            setp = ' '.join(['%s = %s_;' % (name, name) \
                             for name in self._prms])
            # function object:
            s = """
class %(function_name)s
{
  // parameters:
  %(decl)s
 public:
  %(function_name)s (%(prms)s)
    { %(setp)s }
  double operator() const (%(varlist)s)
    { return %(expr)s; }
};
""" % vars()
        else:
            s = """
double %(function_name)s (%(varlist)s)
{ return %(expr)s; }
"""
        return s

    def F77_code(self, function_name='somefunc'):
        """
        Dump the string expressions as a Fortran 77 function or subroutine.

        Note: if pow(x,a) is used in the expression, this is
        translated to x**a by a simple regex, which may fail if
        there are function calls inside pow(.,.).
        """
        expr = self._f
        real = 'real*8'
        varlist = ', '.join(self._var)
        varlist_decl = '\n'.join(['      %s %s' % (real, name) \
                                  for name in self._var])
        decl = '\n'.join(['      %s %s' % (real,name) for name in self._prms])
        import re
        if re.search(r'pow\s*\(', expr):
            # try to replace pow(x,a) by x**a
            expr = re.sub(r'pow\(([^,]+),([^)]+)\)', '((\g<1>)**(\g<2>))',
                          expr)
        # set parameter values:
        setp = '\n'.join(['      %s = %s' % (name, self._prms[name]) \
                          for name in self._prms])
        s = """
      %(real)s function %(function_name)s(%(varlist)s)
C     independent variables:
%(varlist_decl)s
""" % vars()
        if self._prms:
            s += """
C     parameters:
%(decl)s
%(setp)s
""" % vars()
        s += """
      %(function_name)s = %(expr)s

      return
      end
""" % vars()
        return s

    def F77_pow(self):
        """
        Generate an F77 function pow(x,a) (x**a). In some
        string expressions that are to be translated to C/C++,
        pow(x,a) must be used instead of x**a, but pow is not
        defined in F77. This code dumpes the necessary F77 version
        of pow such that string functions can be seamlessly translated
        to C, C++, and F77.

        Note: this function is difficult to use since it expects
        exactly a single-precision power. We now rely on regular
        expressions to replace pow(x,a) by x**a in F77_code instead.
        """
        s = """

      real*8 function pow(x, a)
      real*8 x
C     the power a is usually a number (single precision),
C     note: it cannot be int
      real*4 a
      pow = x**a
      return
      end
"""
        return s

    def C_code(self, function_name='somefunc', inline=False):
        """
        Dump the string expressions as a C function.
        If inline is true, the C++ inline keyword is inserted
        to make the function inline.
        """
        if inline:
            s = 'inline '
        else:
            s = ''
        expr = self._f
        # ** does not work in C, instead of doing sophisticated
        # parsing and edit, provide an error message
        self._pow_check()

        varlist = ', '.join(['double %s' % var for var in self._var])
        s += 'double %s (%s)\n{\n' % (function_name, varlist)
        if self._prms:
            # declare variables for parameters:
            decl = ' '.join(['double %s;' % name for name in self._prms])
            # set parameter values:
            prms = ' '.join(['%s = %s;' % (name, self._prms[name]) \
                            for name in self._prms])
            s += '  %s\n  %s\n' % (decl, prms)
        # evaluate math expression:
        s += '  return ' + expr + ';\n}\n'
        return s

    def _pow_check(self):
        """
        Raise a SyntaxError exception if the ** power operator is used
        in the string formula.
        """
        if self._f.find('**') != -1:
            raise SyntaxError, \
                  'use pow(a,b) instead of a**b in the expression'\
                  '\n%s\n(since you demand translation to C/C++)' % self._f

def _doctest():
    import doctest, StringFunction
    return doctest.testmod(StringFunction)

def _demo():
    f = StringFunction('a+b*sin(x)', a=1, b=4)
    print f(2)
    f.set_parameters(a=-1, b=pi)
    print f(1)
    print 'internals:', str(f), repr(f), f._lambda, f._prms
    f = StringFunction('amp*sin(a*t)*exp(-6.211*x)',
                       independent_variables=('x','t'))
    f.set_parameters(amp=0.1, a=1)
    print f(0,pi/2.0,a=2)
    print 'internals:', str(f), repr(f), f._lambda, f._prms
    print f.C_code()
    print f.Cpp_code()
    print f.F77_code()


# simplified "pedagogical" versions from the
# "Python for Computational Science" book:

class StringFunction_v1:
    def __init__(self, expression):
        self._f = expression

    def __call__(self, x):
        return eval(self._f)  # evaluate function expression

# compile eval expression:
class StringFunction_v2:
    def __init__(self, expression):
        self._f_compiled = compile(expression, '<string>', 'eval')

    def __call__(self, x):
        return eval(self._f_compiled)

# allow parameters and an arbitrary name of the independent variable:
class StringFunction_v3:
    def __init__(self, expression,
                 independent_variable='x',
                 set_parameters=''):
        self._f_compiled = compile(expression, '<string>', 'eval')
        self._var = independent_variable  # 'x', 't' etc.
        self._code = set_parameters
        self.__name__ = expression  # name of func is expression

    def set_parameters(self, code):
        self._code = code

    def __call__(self, x):
        # assign value to independent variable:
        exec '%s = %g' % (self._var, x)
        # execute some user code (defining parameters etc.):
        if self._code:  exec(self._code)
        return eval(self._f_compiled)

# let parameters be keyword arguments:
class StringFunction_v4:
    def __init__(self, expression, **kwargs):
        self._f = expression
        self._var = kwargs.get('independent_variable', 'x') # 'x', 't' etc.
        self._globals = kwargs.get('globals', globals())
        self.__name__ = self._f  # class name = function expression
        self._f_compiled = compile(self._f, '<string>', 'eval')
        self._prms = kwargs.copy()
	try:
            del self._prms['independent_variable']
            del self._prms['globals']
        except: pass

    def set_parameters(self, **kwargs):
        self._prms.update(kwargs)

    def __call__(self, x):
        # include indep. variable in dictionary of function parameters:
        self._prms[self._var] = x
        # evaluate function expression:
        return eval(self._f_compiled, self._globals, self._prms)

    def test(self, x=1.4325):
        # test that all parameters are defined
        try:
            self(x)  # sample call
            return True
        except NameError, e:
            prm = str(e).split()[2]
            raise NameError, 'Parameter "%s" is not defined,\nneither in '\
                  'the constructor nor set_parameters.\n'\
                  'Update the constructor call or call set_parameters'\
                  '(%s=...)' % (prm, prm)
        else:
            return True  # accept other errors

    def __str__(self):
        s = self._f
        # Substitute parameter names by their numerical values.
        # Start with the most complicated parameter names
        # (this algorithm may fail).

        # first remove indep. variables possibly inserted in self._prms
        # by the self.__call__ method:
        try:
            del self._prms[self._var]
        except:
            pass
        prm_names = self._prms.keys()
        prm_names.sort(lambda a, b: cmp(len(a), len(b)))
        prm_names.reverse()
        for name in prm_names:
            s = s.replace(name, str(self._prms[name]))
        return s

    def __repr__(self):
        # first remove indep. variables possibly inserted in self._prms
        # by the self.__call__ method:
        try:
            del self._prms[self._var]
        except:
            pass
        kwargs = ', '.join(['%s=%s' % (key, repr(value)) \
                            for key, value in self._prms.items()])
        return """StringFunction1(%s, independent_variable=%s, %s)""" % \
               (repr(self._f), repr(self._var), kwargs)

class StringFunction_v5(StringFunction_v4):
    """
    Extension of class StringFunction_v4 to an arbitrary
    number of independent variables.
    """
    def __init__(self, expression, **kwargs):
        StringFunction_v4.__init__(self, expression, **kwargs)
        self._var = tuple(kwargs.get('independent_variables', 'x'))
        try:    del self._prms['independent_variables']
        except: pass

    def __call__(self, *args):
        # add independent variables to self._prms:
        for name, value in zip(self._var, args):
            self._prms[name] = value
        return eval(self._f_compiled, self._globals, self._prms)

    def __str__(self):
        # remove the independent variables from self._prms such that
        # this dict contains parameters (to be subsituted by values) only:
        try:
            for v in self._var:
                del self._prms[v]
        except:
            pass
        return StringFunction1.__str__(self)


    def __repr__(self):
        # first remove indep. variables possibly inserted in self._prms
        # by the self.__call__ method:
        try:
            for v in self._var:
                del self._prms[v]
        except:
            pass
        kwargs = ', '.join(['%s=%s' % (key, repr(value)) \
                            for key, value in self._prms.items()])
        return """StringFunction(%s, independent_variables=%s, %s)""" % \
               (repr(self._f), repr(self._var), kwargs)



def _efficiency():
    print '\nPerform some efficiency tests (this might take some time...):'
    formula = 'sin(x) + x**3 + 2*x'
    formula_wprm = formula + ' + A*B'
    def s0(x):
        return sin(x) + x**3 + 2*x
    s1 = StringFunction_v1(formula)
    s2 = StringFunction_v2(formula)  # compiled
    s3 = StringFunction_v3(formula_wprm, set_parameters='A=0; B=0')
    s4 = StringFunction_v4(formula)
    s5 = StringFunction_v5(formula, independent_variables=('x',),
                           A=0, B=0)
    s6 = StringFunction(formula, independent_variables=('x',),
                        A=0, B=0)
    s7 = s6.__call__
    x = 0.9
    # verification first:
    values = [s(x) for s in s0, s1, s2, s3, s4, s5, s6, s7]
    print 'values of %s for x=%s: %s' % (formula, x, values)

    n = 400000
    from scitools.misc import timer
    from scitools.EfficiencyTable import EfficiencyTable as ET
    e = ET('Efficiency check of StringFunction implementations; '
           'formula=%s, n=%d' % (formula, n))
    import inspect
    for s in s0, s1, s2, s3, s4, s5, s6, s7:
        if inspect.isfunction(s) or inspect.ismethod(s):
            name = s.__name__
        else:
            name = s.__class__.__name__
        t = timer(s, args=(x,), repetitions=100000,
                  comment=name)
        e.add(name, t)
    print e
    print 'End of efficiency tests\n'

if __name__ == '__main__':
    _doctest()
    _demo()
    _efficiency()



#!/usr/bin/env python
"""
Read tabular data from file into NumPy arrays and vice versa.

This module provides functions for
1) reading row-column table data from file into NumPy arrays, and
2) writing two-dimensional NumPy arrays to file in a table fashion.

  - read: Load a table with numbers into a two-dim. NumPy array.

  - write: Write a two-dim. NumPy array a in tabular form.

  - read_columns:
    As read, but the columns are returned as separate arrays instead
    of a two-dimensional array.

  - write_columns:
    As write, but the arguments are comma-separated one-dimensional
    arrays, one for each column, instead of a two-dimensional array.

The file format requires the same number of "words" (numbers)
on each line. Comment lines are allowed, but a blank line
indicates a delimiter in the data set, and lines after the blank
line will not be read.

Example: Assume we have a data file `tmp.dat` with the numbers::

  0        0.0        0.0        1.0
  1        1.0        1.0        2.0
  2        4.0        8.0       17.0
  3        9.0       27.0       82.0
  4       16.0       64.0      257.0
  5       25.0      125.0      626.0

The following session demonstrates key functions in this module::

    >>> import scitools.filetable as ft
    >>> s = open('tmp.dat', 'r')
    >>> table = ft.read(s)
    >>> s.close()
    >>> print table
    [[   0.    0.    0.    1.]
     [   1.    1.    1.    2.]
     [   2.    4.    8.   17.]
     [   3.    9.   27.   82.]
     [   4.   16.   64.  257.]
     [   5.   25.  125.  626.]]

    >>>
    >>> s = open('tmp.dat', 'r')
    >>> x, y1, y2, y3 = ft.read_columns(s)
    >>> s.close()
    >>> print x
    [ 0.  1.  2.  3.  4.  5.]
    >>> print y1
    [  0.   1.   4.   9.  16.  25.]
    >>> print y2
    [   0.    1.    8.   27.   64.  125.]
    >>> print y3
    [   1.    2.   17.   82.  257.  626.]
    >>>
    >>> s = open('tmp2.dat','w')
    >>> ft.write(s, table)
    >>> s.close()

The `tmp2.dat` file looks as follows::

    0       0       0       1
    1       1       1       2
    2       4       8       17
    3       9       27      82
    4       16      64      257
    5       25      125     626

"""
# author: Hans Petter Langtangen <hpl@ifi.uio.no>

import sys, os, re
from numpy import *

__all__ = ['read', 'read_columns', 'readfile',
           'write', 'write_columns',]

# simple version (not as effective as function read):
def read_v1(fileobj, commentchar='#'):
    """Load a table with numbers into a two-dim. NumPy array."""
    # read until next blank line:
    r = []  # total set of numbers (r[i]: numbers in i-th row)
    while True:  # might call read several times for a file
        line = fileobj.readline()
        if not line: break  # end of file
        if line.isspace(): break  # blank line
        if line[0] == commentchar: continue # treat next line
        r.append([float(s) for s in line.split()])
    return array(r)


def read(fileobj, commentchar='#'):
    """
    Load a table with numbers into a two-dim. NumPy array.
    @param fileobj: open file object.
    @param commentchar: lines starting with commentchar are skipped
    (a blank line is an array data delimiter and stops reading).
    @return: two-dimensional (row-column) NumPy array.
    """
    # based on a version by Mario Pernici <Mario.Pernici@mi.infn.it>
    location = fileobj.tell()
    import re
    commentchar = re.escape(commentchar)
    while True:
        line = fileobj.readline()
        if not line: break  # end of file
        elif line.isspace(): break  # blank line
        elif re.match(commentchar, line): continue # treat next line
        else: break

    shape1 = len(line.split())
    if shape1 == 0: return None
    fileobj.seek(location)

    blankline = re.compile('\n\s*\n',  re.M)
    commentline = re.compile('^%s[^\n]*\n' % commentchar, re.M)
    filestr = fileobj.read()
    # remove lines after a blank line
    m = re.search(blankline, filestr)
    if m:
        filestr = filestr[:m.start()+1]
    # skip lines starting with the comment character
    filestr = re.sub(commentline, '', filestr)
    a = [float(x) for x in filestr.split()]
    data = array(a)
    data.shape = (len(a)/shape1, shape1)
    return data


def read_columns(fileobj, commentchar='#'):
    """As read. Return columns as separate arrays."""
    a = read(fileobj, commentchar)
    return [a[:,i] for i in range(a.shape[1])]

# for backward compatibility:
def readfile(filename, commentchar='#'):
    """
    As read, but a filename (and not a file object) can be given.
    Return: columns as separate arrays.
    """
    f = open(filename, 'r')
    a = read(f, commentchar)
    r = [a[:,i] for i in range(a.shape[1])]
    return r



# simple write version:
def write_v1(fileobj, a):
    """Write a two-dim. NumPy array a in tabular form to fileobj."""
    if len(a.shape) != 2:
        raise TypeError("a 2D array is required, shape now is "+str(a.shape))
    for i in range(a.shape[0]):
        for j in range(a.shape[1]):
            fileobj.write(str(a[i,j]) + "\t")
        fileobj.write("\n")

# faster write version:
def write_v2(fileobj, a):
    """Write a two-dim. NumPy array a in tabular form to fileobj."""
    # written by Mario Pernici <Mario.Pernici@mi.infn.it>
    if a.shape[1] == 1:
        for i in xrange(a.shape[0]): fileobj.write('%g\n' % a[i,0])
    elif a.shape[1] == 2:
        for i in xrange(a.shape[0]): fileobj.write('%g\t%g\n' % \
                                                (a[i,0],a[i,1]))
    elif a.shape[1] == 3:
        for i in xrange(a.shape[0]): fileobj.write('%g\t%g\t%g\n' % \
                                                (a[i,0],a[i,1],a[i,2]))
    else:
        str_fmt = '%g\t'*(a.shape[1] - 1) + '%g\n'
        for i in xrange(a.shape[0]):
            fileobj.write(str_fmt % tuple(a[i]))

    # don't think about
    # out = str(a).replace('[','').replace(',', '\t')
    # fileobj.write(out + '\n')
    # the str(a) conversion is *extremely* expensive

def write_v3(fileobj, a):
    """Short and fast version, compared to write_v1 and write_v2."""
    # written by Mario Pernici <Mario.Pernici@mi.infn.it>
    fileobj.write(('%g\t'*(a.shape[1]-1) + '%g\n')*a.shape[0] % tuple(ravel(a)))

def write(fileobj, a):
    """Write a two-dim. NumPy array a in tabular form to fileobj."""
    # fastest version (of the write family of functions) so far...
    # written by Mario Pernici <Mario.Pernici@mi.infn.it>
    
    if len(a.shape) != 2:
        raise TypeError("a 2D array is required, shape now is "+str(a.shape))
    N = 512
    shape0 = a.shape[0]
    shape1 = a.shape[1]
    str_fmt = '%g\t'*(shape1 - 1) + '%g\n'
    # use a big format string
    str_fmt_N = str_fmt * N
    for i in xrange(shape0/N):
      a1 = a[i:i+N,:]
      # put a1 in  1D array form; ravel better than reshape for
      # non-contiguous arrays.
      a1 = ravel(a1)
      fileobj.write(str_fmt_N % tuple(a1))
    for i in range(shape0 - shape0%N, shape0):
      fileobj.write(str_fmt % tuple(a[i]))


def write_columns(fileobj, *columns):
    """
    As write, but the column data are represented as one-dimensional
    arrays.
    """
    a = array(columns).transpose()
    write(fileobj, a)
    
    
# testing:
def _generate(nrows, ncolumns, filename):
    f = open(filename, 'w')
    a = fromfunction(lambda i,j: i+j*j, (nrows,ncolumns))
    narrays = 3  # no of arrays in the file
    f.write('# %d arrays in this file\n#\n' % narrays)
    for i in range(narrays):
        f.write('# a new array with %d rows:  \n' % nrows)
        f.write('#\n')
        write(f, a)
        f.write('\n')  # array delimiter

def _test(n):
    _generate(n, 3, "tmp.1")
    import time
    t0 = time.clock()
    f = open("tmp.1", "r")
    narrays = int(f.readline().split()[1])
    print 'found %d arrays in file' % narrays
    for i in range(narrays):
        q = read(f)
        print 'read an array with shape', q.shape
    t1 = time.clock()
    print "read:", t1-t0, "seconds\n"
    f.close()
    t0 = time.clock()
    f = open("tmp.2", "w")
    write(f,q)
    t1 = time.clock()
    print "write:", t1-t0, "seconds"
    
    # compare with TableIO:
    try:
        import TableIO.TableIO as TableIO
    except:
        sys.exit(0) # exit silently
    t0 = time.clock()
    p = TableIO.readColumns("tmp.1", "#")
    #print 'p:', p
    t1 = time.clock()
    print "TableIO.readColumns:", t1-t0, "seconds\n"
    t0 = time.clock()
    TableIO.writeArray("tmp.3", array(p))
    t1 = time.clock()
    print "TableIO.writeArray:", t1-t0,"seconds"

if __name__ == '__main__':
    _test(100000)
    





"""Test functions for plot3 command."""

from easyviztest import *


class test_plot3_basic(EasyvizTestCase):
    def check_plot3_default(self):
        x,y,z = self.get_3D_line_data()
        plot3(x,y,z)
        title("plot3(x,y,z)")
        n()

    def check_plot3_only_z(self):
        x,y,z = self.get_3D_line_data()
        plot3(z)
        title("plot3(z)")
        n()
        
    def check_plot3_kwarg_x_and_y(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,x=x,y=y)
        title("plot3(z,x=x,y=y)")
        n()
        
    def check_plot3_kwarg_auto_x_and_y(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,x='auto',y='auto')
        title("plot3(z,x='auto',y='auto')")
        n()
        
    def check_plot3_two_lines(self):
        x,y,z = self.get_3D_line_data()
        plot3(x,y,z,x,y,-z)
        title("plot3(x,y,z,x,y,-z)")
        n()
        
    def check_plot3_two_lines_kwarg_x_and_y(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,-z,x=x,y=y)
        title("plot3(z,-z,x=x,y=y)")
        n()
        
    def check_plot3_two_lines_kwarg_auto_x_and_y(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,-z,x='auto',y='auto')
        title("plot3(z,-z,x='auto',y='auto')")
        n()

    def check_plot3_multiple_lines(self):
        x,y,z = self.get_3D_line_data()
        plot3(x,y,z,x,y,-z,x,y,2*z,x,y,-2*z)
        title("plot3(x,y,z,x,y,-z,x,y,2*z,x,y,-2*z)")
        n()
        
    def check_plot3_multiple_lines_kwarg_x_and_y(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,-z,2*z,-2*z,x=x,y=y)
        title("plot3(z,-z,2*z,-2*z,x=x,y=y)")
        n()
        
    def check_plot3_multiple_lines_kwarg_auto_x(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,-z,2*z,-2*z,x='auto',y='auto')
        title("plot3(z,-z,2*z,-2*z,x='auto',y='auto')")
        n()


class test_plot3_format_string(EasyvizTestCase):
    def check_plot3_multiple_lines_fmt(self):
        x,y,z = self.get_3D_line_data()
        plot3(x,y,z,'r-',x,y,-z,'b--',x,y,2*z,'g--',x,y,-2*z,'x')
        title("plot3(x,y,z,'r-',x,y,-z,'b--',x,y,2*z,'g--',x,y,-2*z,'x')")
        n()

    def check_plot3_multiple_lines_fmt_kwarg_x(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,'r-',-z,'b--',2*x,'g--',-2*x,'x',x=x,y=y)
        title("plot3(z,'r-',-z,'b--',2*x,'g--',-2*x,'x',x=x,y=y)")
        n()

    def check_plot3_multiple_lines_fmt_kwarg_auto_x(self):
        x,y,z = self.get_3D_line_data()
        plot3(z,'r-',-z,'b--',2*z,'g--',-2*z,'x',x='auto',y='auto')
        title("plot3(z,'r-',-z,'b--',2*z,'g--',-2*z,'x',x='auto',y='auto')")
        n()

    def check_plot3_linecolors(self):
        x,y,z = self.get_3D_line_data()
        setp(show=False)
        colors = Line._colors
        for i,color in enumerate(colors):
            plot3(x,y,z+i,color,hold='on')
        title("different line colors")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_plot3_linestyles(self):
        x,y,z = self.get_3D_line_data()
        setp(show=False)
        styles = Line._linestyles
        for i,style in enumerate(styles):
            plot3(x,y,z+i,style,hold='on')
        title("different line styles")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_plot3_linemarkers(self):
        x,y,z = self.get_3D_line_data()
        setp(show=False)
        markers = Line._markers
        for i,marker in enumerate(markers):
            plot3(x,y,z+i,marker,hold='on')
        title("different line markers")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_plot3_linewidth(self):
        x,y,z = self.get_3D_line_data()
        setp(show=False)
        for i in range(1,10):
            plot3(x,y,z+i,str(i),hold='on')
        title("different line widths")
        hold('off')
        setp(show=screenplot)
        show()
        n()


class test_plot3_with_NaNs(EasyvizTestCase):
    def check_plot3_NaN_in_list(self):
        plot3([0,1,NaN,3,4])
        title("plot3([0,1,NaN,3,4])")
        n()
        
    def check_plot3_NaN_in_array(self):
        plot3(array([0,1,NaN,3,4],float))
        title("plot3(array([0,1,NaN,3,4],float))")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_plot3_basic,'check_'))
        suites.append(unittest.makeSuite(test_plot3_format_string,'check_'))
        suites.append(unittest.makeSuite(test_plot3_with_NaNs,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""
A collection of Python utilities originally developed for the
"Python for Computational Science" book.
"""

import time, sys, os, re, getopt, math, threading, shutil, commands
from errorcheck import right_type
from scitools.StringFunction import StringFunction

def import_module(package, module=None):
    try:
        if module is not None:
            stm = 'from %s import %s as m' % (package, module)
        else:
            stm = 'import %s as m' % (package)
        exec(stm)
        return m

    except ImportError:
        msg = 'Tried %s,\nbut package %s is not installed' % \
              (stm, package.split('.')[0])
        if package.startswith('Scientific'):
            url = '\nDownload from https://sourcesup.cru.fr/projects/scientific-py/'
        elif package.startswith('scipy'):
            url = '\nDownload from http://'
        elif package.startswith('Pmw'):
            url = '\nDownload from http://'
        elif package.startswith('Gnuplot'):
            url = '\nDownload from http://'
        else:
            url = ''
        msg += url
        raise ImportError(msg)


def test_if_module_exists(modulename, msg='',
                          raise_exception=False, abort=True):
    """
    Test if modulename can be imported, and if not, write
    an error message and (optionally) raise an exception, continue or
    abort with sys.exit(1).
    """
    try:
        __import__(modulename)
        return True
    except ImportError:
        import debug
        message = 'Could not import module "%s" - it is '\
                  'not installed on your system. %s\n' % \
                  (modulename, msg)
        if raise_exception:
            if msg:
                print msg
                #print 'The problem arose in ',
                debug.trace(frameno=-3)
            raise ImportError(message)
        else:
            if msg:
                print '\n', message
                #print 'The problem arose in ',
                debug.trace(frameno=-3)
            if abort:
                sys.exit(1)
            else:
                return False
    except Exception, e:
        if msg:
            print msg
            print 'Got an exception while trying to import %s:\n' % \
                modulename, e


def func_to_method(func, class_, method_name=None):
    """
    Add a function to a class class_ as method_name.
    If method_name is not given, func.__name__ becomes
    the name of the method.
    Borrowed from recipe 5.12 in the Python Cookbook.
    """
    setattr(class_, method_name or func.__name__, func)


def system(command, verbose=True, failure_handling='exit', fake=False):
    """
    User-friendly wrapping of the os.system/os.popen commands.
    Actually, the commands.getstatusoutput function is used on Unix
    systems, and the output from the system command is fetched.

    ================  ========================================================
    ================  ========================================================
    command           operating system command to be executed
    verbose           False: no output, True: print command prior to execution
    failure_handling  one of 'exit', 'warning', 'exception', or 'silent'
                      (in case of failure, the output from the command is
                      always displayed)
    fake              if True, the command is printed but not run (for testing)
    return value      the same as commands.getstatusoutput, i.e., a boolean
                      failure variable and the output from the command as a
                      string object
    ================  ========================================================
    """
    if verbose:
        print 'Running operating system command\n   %s' % command
    if fake:
        return 0, 'testing "%s"' % command

    if sys.platform[:3] == 'win':
        result = os.popen(command)
        output = result.read()
        failure = result.close()
    else:
        # Unix/Linux/Mac:
        failure, output = commands.getstatusoutput(command)

    if failure:
        msg = 'Failure when running operating system command'\
              '\n  %s\nOutput:\n%s' % (command, output)
        if failure_handling == 'exit':
            print msg, '\nExecution aborted!'
            sys.exit(1)
        if failure_handling == 'warning':
            print 'Warning:', msg
        elif failure_handling == 'exception':
            raise OSError(msg)
        elif failure_handling == 'silent':
            pass
        else:
            raise ValueError('wrong value "%s" of failure_handling' % \
                             failure_handling)

    return failure, output


def read_cml(option, default=None, argv=sys.argv):
    """
    Search for option (e.g. '-p', '--plotfile') among the command-line
    arguments and return the associated value (the proceeding argument).
    If the option is not found, the default argument is returned
    as str(default) (to have a unified behavior in that everything returned
    from read_cml is a string).

    The call::

       str2obj(read_cml(option, default=...))

    will return a Python object (with the right type) corresponding to
    the value of the object (see the str2obj function).

    ================  ========================================================
    ================  ========================================================
    option            command-line option (str)
    default           default value associated with the option
    argv              list that is scanned for command-line arguments
    return value      the item in argv after the option, or default if option
                      is not found
    ================  ========================================================

    See the read_cml_func function for reading function expressions
    or function/instance names on the command line and returning
    callable objects.
    """
    try:
        index = argv.index(option)
        return argv[index+1]
    except ValueError:
        return str(default)
    except IndexError:
        raise IndexError('array of command-line arguments is too short; '\
                         'no value after %s option' % option)



def str2bool(s):
    """
    Turn a string s, holding some boolean value
    ('on', 'off', 'True', 'False', 'yes', 'no' - case insensitive)
    into boolean variable. s can also be a boolean. Example:

    >>> str2bool('OFF')
    False
    >>> str2bool('yes')
    True
    """
    if isinstance(s, str):
        true_values = ('on', 'true', 'yes')
        false_values = ('off', 'false', 'no')
        s2 = s.lower()  # make case insensitive comparison
        if s2 in true_values:
            return True
        elif s2 in false_values:
            return False
        else:
            raise ValueError('"%s" is not a boolean value %s' % \
                             (s, true_values+false_values))
    else:
        raise TypeError('%s %s cannot be converted to bool' % \
                        (s, type(s)))


def str2obj(s, globals_=None, locals_=None, debug=False):
    """
    Turn string s into the corresponding object. str2obj is mainly
    used to take a string from a GUI or the command line and
    create a Python object. For example:

    >>> s = str2obj('0.3')
    >>> print s, type(s)
    0.3 <type 'float'>
    >>> s = str2obj('(1,8)')
    >>> print s, type(s)
    (1, 8) <type 'tuple'>

    Method: eval(s) can normally do the job, but if s is meant to
    be turned into a string object, eval works only if s has explicit
    quotes:

    >>> eval('some string')
    Traceback (most recent call last):
    SyntaxError: unexpected EOF while parsing

    (eval tries to parse 'some string' as Python code.)
    Similarly, if s is a boolean word, say 'off' or 'yes',
    eval will not work.

    In this function we first try to see if s is a boolean value,
    using scitools.misc.str2bool. If this does is not successful,
    we try eval(s, globals_, locals_), and if it works, we return
    the resulting object. Otherwise, s is (most probably) a string,
    so we return s itself. The None value of locals_ and globals_
    implies using locals() and globals() in this function.

    Examples:

    >>> strings = ('0.3', '5', '[-1,2]', '-1+3j', 'dict(a=1,b=0,c=2)',
    ...            'some string', 'true', 'ON', 'no')
    >>> for s in strings:
    ...     obj = str2obj(s)
    ...     print '"%s" -> %s %s' % (s, obj, type(obj)
    ...
    "0.3" -> 0.3 <type 'float'>
    "5" -> 5 <type 'int'>
    "[-1,2]" -> [-1, 2] <type 'list'>
    "-1+3j" -> (-1+3j) <type 'complex'>
    "dict(a=1,b=0,=2)" ->  {'a': 1, 'c': 2, 'b': 0} <type 'dict'>
    "some string" -> some string <type 'str'>
    "true" -> True <type 'bool'>
    "ON" -> True <type 'bool'>
    "no" -> False <type 'bool'>
    >>>

    If the name of a user defined function, class or instance is
    sent to str2obj, the calling code must also send locals() and
    globals() dictionaries as extra arguments. Otherwise, str2obj
    will not know how to "eval" the string and produce the right
    object (user-defined types are unknown inside str2obj unless
    the calling code's globals and locals are provided).
    Here is an example:

    >>> def myf(x):
    ...     return 1+x
    ...
    >>> class A:
    ...     pass
    ...
    >>> a = A()
    >>>
    >>> s = str2obj('myf')
    >>> print s, type(s)   # now s is simply the string 'myf'
    myf <type 'str'>
    >>> # provide locals and globals such that we get the function myf:
    >>> s = str2obj('myf', locals(), globals())
    >>> print s, type(s)
    <function myf at 0xb70ffe2c> <type 'function'>
    >>> s = str2obj('a', locals(), globals())
    >>> print s, type(s)
    <__main__.A instance at 0xb70f6fcc> <type 'instance'>

    With debug=True, the function will print out the exception
    encountered when doing eval(s, globals_, locals_), and this may
    point out problems with, e.g., imports in the calling code
    (insufficient variables in globals_).

    Note: if the string argument is the name of a valid Python
    class (type), that class will be returned. For example,
    >>> str2obj('list')  # returns class list
    <type 'list'>
    """
    if globals_ is None:
        globals_ = globals()
    if locals_ is None:
        locals_ = locals()

    try:
        b = str2bool(s)
        return b
    except (ValueError, TypeError), e:
        # s is not a boolean value, try eval(s):
        try:
            b = eval(s, globals_, locals_)
            return b
        except Exception, e:
            if debug:
                print """
scitools.misc.str2obj:
Tried to do eval(s) with s="%s", and it resulted in an exception:
    %s
""" % (s, e)
            # eval(s) did not work, s is probably a string:
            return s


def str2type(value):
    """
    Return a function that can take a string and convert it to
    a Python object of the same type as value.

    This function is useful when turning input from GUIs or the
    command line into Python objects. Given a default value for the
    input (with the right object type), str2type will return the right
    conversion function.  (str2obj can do the thing, but will often
    return eval, which turns any string into a Python object - this is
    less safe than str2type, which never returns eval. That principle
    helps to detect wrong input.)

    Method: If value is bool, we use scitools.misc.str2bool, which
    is capable of converting strings like "on", "off","yes", "no",
    "true" and "false" to boolean values. Otherwise, we use
    type(value) as the conversion function. However, there is one
    problem with type(value) when value is an int while the user
    intended a general real number - in that case one may get
    wrong answers because of wrong (int) round off. Another problem
    concerns user-defined types. For those (which str2type knows
    nothing about) the str function is returned, implying that the
    conversion from a string to the right user-defined type cannot
    be done by the function returned from str2type.

    Examples:

    >>> f = str2type((1,4,3))
    >>> f.__name__
    'tuple'

    >>> f = str2type(MySpecialClass())
    >>> f.__name__
    'str'

    >>> f = str2type('some string')
    >>> f.__name__
    'str'

    >>> f = str2type(False)
    >>> f.__name__
    'str2bool'

    (Note that we could return eval if value is not a string or a boolean,
    but eval is never returned from this function to avoid conversion
    to an unintended type.)
    """
    if isinstance(value, bool):
        return str2bool
    elif isinstance(value, (basestring,int,float,complex,list,tuple,dict)):
        return type(value)
    else:
        # the type of value is probably defined in some unknown module
        return str


def interpret_as_callable_or_StringFunction(
    s, iv, globals_, **named_parameters):
    """
    Return a callable object if ``s`` is the name of such an
    object, otherwise turn ``s`` to a ``StringFunction`` object with
    ``iv`` as the name of the independent variable.
    The ``named_parameters`` dictionary holds parameters in
    string expressions.
    Used by the ``read_cml`` function.

    """
    if isinstance(globals_, dict):
        # First assume s is a user defined function or instance creation
        if isinstance(s, str):
            try:
                obj = eval(s, globals_)
                if callable(obj):
                    return obj
                else:
                    s_is_string = True
            except NameError:
                s_is_string = True
        else:
            if callable(s):  # user function obj or lambda func obj
                return s

    elif globals_ is None:
        # No global names supplied, s cannot be the string of a
        # user function or instance
        if callable(s):
            return s
        else:
            s_is_string = True
    else:
        raise ValueError('globals_ must dict or None')

    if s_is_string:
        # Assume that s is a string expression
        if isinstance(iv, str):  # single indep. variable?
            iv = [iv]
        func = StringFunction(s, independent_variables=iv,
                              globals=globals_, **named_parameters)
        return func
    else:
        # Should never come here
        raise ValueError('s is neither a string expression, nor a callable %s' % type(s))



def read_cml_func(option, default_func, iv='t', globals_=None,
                  **named_parameters):
    """
    Locate ``--option`` on the command line (``sys.argv``) and find
    the corresponding value (next ``sys.argv`` element).
    This value is supposed to specify a Python function, an
    instance with a ``__call__`` method, None, or a string that can be
    turned into a ``scitools.StringFunction.StringFunction``
    function with ``iv`` as the name of the independent variable(s)
    (list of strings in case of more than one independent variable).
    If ``--option`` is not found, the argument ``default_func``,
    a given callable or string, is returned (if string, ``iv``
    reflects the name of the independent variable(s) in the
    string).

    The ``globals_`` argument is just passed on to the
    ``StringFunction`` object if the value of the option for default
    function is a string. Similary, the ``named_parameters`` dictionary
    is passed on to the ``StringFunction`` object and assumed to
    hold parameters in the string expressions (variables different
    from the independent variable).

    This function always returns a callable object or None.

    Here is an interactive session showing the use of ``read_cml_func``::

    >>> from scitools.misc import read_cml_func
    >>> import sys
    >>> from math import sin, cos, pi
    >>>
    >>> # fake command-line arguments by filling in sys.argv:
    >>> sys.argv[1:] = '--func1 myfunc --func3 sin(x)'.split()
    >>>
    >>> def myfunc(x):
    ...     return 1 + x
    ...
    >>>
    >>> # --func1 has myfunc as value, must pass on globals() or just
    >>> # myfunc as name to read_cml_func
    >>> #f = read_cml_func('--func1', '1', globals_=globals())
    >>> f = read_cml_func('--func1', '1', globals_={'myfunc': myfunc})
    >>> type(f)
    <type 'function'>
    >>> f(10)
    11
    >>> # --func3 is given as a string expression "sin(x)" on the command line
    >>> f = read_cml_func('--func3', '0', iv='x')
    >>> type(f)
    <type 'instance'>
    >>> repr(f)
    "StringFunction('sin(x)', independent_variables=('x',), )"
    >>> str(f)
    'sin(x)'
    >>> f(pi)
    1.2246467991473532e-16
    >>>
    >>> # --func2 is not given on the command line, use the default
    >>> # value "A*cos(w*t)", which is a string expression.
    >>> # Pass on a globals_ dict with cos from numpy such that f works
    >>> # with array argument for t
    >>> import numpy
    >>> f = read_cml_func('--func2', 'A*cos(w*t)', iv='t', A=3, w=pi, globals_={'cos': numpy.cos})
    >>> # More general case where the string should have all numpy functions:
    >>> #f = read_cml_func('--func2', 'A*cos(w*t)', iv='t', A=3, w=pi, globals_=dir(numpy))
    >>> type(f)
    <type 'instance'>
    >>> repr(f)
    "StringFunction('A*cos(w*t)', independent_variables=('t',), A=3, w=3.141592653589793)"
    >>> str(f)
    '3*cos(3.14159265359*t)'
    >>> t = numpy.array([1, 4])
    >>> f(t)
    array([-3.,  3.])
    >>>
    >>> # No --func4 on the command line (sys.argv), implying that
    >>> # f becomes a StringFunction with value 0
    >>> f = read_cml_func('--func4', '0')
    >>> type(f)
    <type 'instance'>
    >>> repr(f)
    "StringFunction('0', independent_variables=('t',), )"
    >>> str(f)
    '0'
    >>> f(1)
    0
    """
    if option in sys.argv:
        i = sys.argv.index(option)
        try:
            value = sys.argv[i+1]
        except IndexError:
            raise IndexError(
                'no value after option %s on the command line' \
                % option)
        return interpret_as_callable_or_StringFunction(
            value, iv, globals_=globals_, **named_parameters)
    else:
        if default_func is None:
            return None
        else:
            # Assume default_func is string expression or a
            # callable function listed in the globals_ dict
            # or just a lambda function
            return interpret_as_callable_or_StringFunction(
                default_func, iv, globals_=globals_, **named_parameters)



def function_UI(functions, argv, verbose=True):
    """
    User interface for calling a collection of functions from the
    command line by writing the function name and its arguments.
    functions is a list of possible functions to be called. argv is
    sys.argv from the calling code.
    function_UI returns a command to be evaluated (function call)
    in the calling code.

    This function autogenerates a user interface to a module.
    Suppose a module has a set of functions::

      test_mymethod1(M, q, a, b)
      test_method2a(a1, a2, a3=0, doc=None)
      test_method2b()

    The following code automatically creates a user interface
    and executes calls to the functions above::

      from scitools.misc import function_UI
      function_names = [fname for fname in dir() if fname.startswith('test_')]
      function_UI(function_names, sys.argv)

    On the command line the user can now type::

      programname --help

    and automatically get a help string for each function, consisting
    of the function name, all its positional arguments and all its
    keyword arguments.

    Alternatively, writing just the function name::

      programname functionname

    prints a usage string if this function requires arguments, otherwise
    the function is just called.

    Finally, when arguments are supplied::

      programname functionname arg1 arg2 arg3 ...

    the function is called with the given arguments. Safe and easy use
    is ensured by always giving keyword arguments::

      programname functionname arg1=value1 arg2=value2 arg3=value3 ...

    """
    usage, doc = _function_args_doc(functions)

    def all_usage():
        for fname in sorted(usage):
            print fname, ' '.join(usage[fname])

    # call: function-name arg1 arg2 ...
    if len(argv) < 2:
        all_usage()
        sys.exit(1)

    function_names = [f.__name__ for f in functions]
    if len(argv) == 2 and argv[1] in function_names and usage[argv[1]]:
        fname = argv[1]
        print 'Usage:', fname, ' '.join(usage[fname])
        if doc[fname]:
            print '\nDocstring:\n', doc[fname]
        sys.exit(1)

    if len(argv) == 2 and argv[1].endswith('help'):
        all_usage()
        sys.exit(0)

    cmd = '%s(%s)' % (argv[1], ', '.join(argv[2:]))
    #if len(argv[2:]) == len(usage[fname]):
        # Correct no arguments (eh, can leave out keyword args...)
    if verbose:
        print 'Calling', cmd
    return cmd


def _function_args_doc(functions):
    """
    Create documentation of a list of functions.
    Return: usage dict (usage[funcname] = list of arguments, incl.
    default values), doc dict (doc[funcname] = docstring (or None)).
    Called by function_UI.
    """
    import inspect
    usage = {}
    doc = {}
    for f in functions:
        args = inspect.getargspec(f)
        if args.defaults is None:
            # Only positional arguments
            usage[f] = args.args
        else:
            # Keyword arguments too, build complete list
            usage[f.__name__] = args.args[:-len(args.defaults)] + \
                     ['%s=%s' % (a, d) for a, d in \
                      zip(args.args[-len(args.defaults):], args.defaults)]
        doc[f.__name__] = inspect.getdoc(f)
    return usage, doc


def before(string, character):
    """Return part of string before character."""
    for i in range(len(string)):
        if c == character:
            return string[:i-1]

def after(string, character):
    """Return part of string after character."""
    for i in range(len(string)):
        if c == character:
            return string[i+1:]

def remove_multiple_items(somelist):
    """
    Given some list somelist, return a list where identical items
    are removed.
    """
    right_type(somelist, list)
    new = []
    helphash = {}
    for item in somelist:
        if not item in helphash:
            new.append(item)
            helphash[item] = 1
    return new


def find(func, rootdir, arg=None):
    """
    Traverse the directory tree rootdir and call func for each file.
    arg is a user-provided argument transferred to func(filename,arg).
    """
    files = os.listdir(rootdir)  # get all files in rootdir
    files.sort(lambda a,b: cmp(a.lower(),b.lower()))
    for file in files:
        fullpath = os.path.join(rootdir,file) # make complete path
        if os.path.islink(fullpath):
            pass # drop links...
        elif os.path.isdir(fullpath):
            find(func, fullpath, arg) # recurse into directory
        elif os.path.isfile(fullpath):
            func(fullpath, arg) # file is regular, apply func
        else:
            print 'find: cannot treat ', fullpath


def sorted_os_path_walk(root, func, arg):
    """
    Like os.path.walk, but directories and files are visited and
    listed in alphabetic order.
    """
    try:
        files = os.listdir(root)  # get all files in rootdir
    except os.error:
        return
    files.sort(lambda a,b: cmp(a.lower(), b.lower()))
    func(arg, root, files)
    for name in files:
        name = os.path.join(root, name)
        if os.path.isdir(name):
            sorted_os_path_walk(name, func, arg) # recurse into directory


def subst(patterns, replacements, filenames,
          pattern_matching_modifiers=0):
    """
    Replace a set of patterns by a set of replacement strings (regular
    expressions) in a series of files.
    The function essentially performs::

      for filename in filenames:
          file_string = open(filename, 'r').read()
          for pattern, replacement in zip(patterns, replacements):
              file_string = re.sub(pattern, replacement, file_string)

    A copy of the original file is taken, with extension `.old~`.

    ==========================  ======================================
    ==========================  ======================================
    patterns                    string or list of strings (regex)
    replacements                string or list of strings (regex)
    filenames                   string or list of strings
    pattern_matching_modifiers  re.DOTALL, re.MULTILINE, etc., same
                                syntax as for re.compile
    ==========================  ======================================
    """
    # if some arguments are strings, convert them to lists:
    if isinstance(patterns, basestring):
        patterns = [patterns]
    if isinstance(replacements, basestring):
        replacements = [replacements]
    if isinstance(filenames, basestring):
        filenames = [filenames]

    # pre-compile patterns:
    cpatterns = [re.compile(pattern, pattern_matching_modifiers) \
                 for pattern in patterns]
    modified_files = dict([(p,[]) for p in patterns])  # init
    messages = []   # for return info

    for filename in filenames:
        if not os.path.isfile(filename):
            raise IOError('%s is not a file!' % filename)
        f = open(filename, 'r');
        filestr = f.read()
        f.close()

        for pattern, cpattern, replacement in \
            zip(patterns, cpatterns, replacements):
            if cpattern.search(filestr):
                filestr = cpattern.sub(replacement, filestr)
                shutil.copy2(filename, filename + '.old~') # backup
                f = open(filename, 'w')
                f.write(filestr)
                f.close()
                modified_files[pattern].append(filename)

    # make a readable return string with substitution info:
    for pattern in sorted(modified_files):
        if modified_files[pattern]:
            replacement = replacements[patterns.index(pattern)]
            messages.append('%s replaced by %s in %s' % \
                                (pattern, replacement,
                                 ', '.join(modified_files[pattern])))

    return ', '.join(messages) if messages else 'no substitutions'


# class Command has now been replaced by the standard functools.partial
# function in Python v2.5 and later:

class Command:
    """
    Alternative to lambda functions.

    This class should with Python version 2.5 and later be replaced
    by functools.partial.
    However, you cannot simply do a::

      Command = functools.partial

    to be backward compatible with your old programs that use Command,
    because Command and functools.partial supply the positional
    arguments in different manners: Command calls the underlying
    function with new arguments followed by the originally recorded
    arguments, while functools.partial does it the other way around
    (first original arguments, then new positional arguments).

    This Command class is kept for backward compatibility. New usage
    should employ functools.partial instead.
    """

    def __init__(self, func, *args, **kwargs):
        self.func = func
        self.args = args
        self.kwargs = kwargs

    def __call__(self, *args, **kwargs):
        args = args + self.args
        self.kwargs.update(kwargs)
        self.func(*args, **self.kwargs)


def timer(func, args=[], kwargs={}, repetitions=10, comment=''):
    """
    Run a function func, with arguments given by the tuple
    args and keyword arguments given by the dictionary kwargs,
    a specified number of times (repetitions) and
    write out the elapsed time and the CPU time together.
    """
    t0 = time.time();  c0 = time.clock()
    for i in range(repetitions):
        func(*args, **kwargs)
    cpu_time = time.clock()-c0
    elapsed_time = time.time()-t0
    try:    # instance method?
        name = func.im_class.__name__ + '.' + func.__name__
    except: # ordinary function
        try:
            name = func.__name__
        except:
            name = ''
    print '%s %s (%d calls): elapsed=%g, CPU=%g' % \
          (comment, name, repetitions, elapsed_time, cpu_time)
    return cpu_time/float(repetitions)


def timer_system(command, comment=''):
    """
    Run an os.system(command) statement and measure the CPU time.
    With os.system, the CPU time is registered as the user and
    system time of child processes.

    Note: there might be some overhead in the timing compared to
    running time in the OS instead.
    """
    t0 = os.times()
    failure = os.system(command)
    t1 = os.times()
    # some programs return nonzero even when they work (grep, for inst.)
    if failure:
        print 'Note: os.system(%s) failed' % command, 'returned', failure
    cpu_time = t1[2]-t0[2] + t1[3]-t0[3]
    print '%s system command: "%s": elapsed=%g CPU=%g' % \
          (comment, command, t1[4]-t0[4], cpu_time)
    return cpu_time


def findprograms(programs, searchlibs=[], write_message=False):
    """
    Given a list of programs (programs), find the full path
    of each program and return a dictionary with the program
    name as key and the full path as value. The value is None
    if the program is not found.

    The program list can either be a list/tuple or a
    dictionary (in the latter case, the keys are the programs
    and the values are explanations of the programs).
    If write_message is true, the function writes a message
    if a program is not found. In that case, None is returned
    if not all programs are found.

    A single program can also be given as first argument. In that
    case, findprograms returns True or False according to whether
    the program is found or not.

    Example on usage::

      if findprograms('plotmtv'):
          os.system('plotmtv ...')

      # write a message if a program is not found:
      if findprograms(['plotmtv'], write_message=True):
          os.system('plotmtv ...')

      programs = ['gs', 'convert']
      path = findprograms(programs)
      if path['gs']:
          os.system('gs ...')
      if path['convert']:
          os.system('convert ...')

      programs = { 'gs' : 'Ghostscript: file format conversions',
                   'convert' : 'File format conversion from ImageMagick',
                 }
      if not findprograms(programs, write_message=True):
          print 'the mentioned programs need to be installed'
          sys.exit(1)
    """
    def program_exists(fullpath):
        if sys.platform.startswith('win'):
            # add .exe or .bat to program filename:
            if os.path.isfile(fullpath+'.exe') or \
               os.path.isfile(fullpath+'.bat'):
                return True
        elif os.name == 'posix': # Unix
            if os.path.isfile(fullpath):
                return True
        else:
            raise TypeError('platform %s/%s not supported' % \
                            (sys.platform, os.name))
        return False # otherwise

    path = os.environ['PATH']  # /usr/bin:/usr/local/bin:/usr/X11/bin
    paths = re.split(os.pathsep, path)
    fullpaths = {}
    if isinstance(programs, str):
        program = programs
        for dir in paths:
            if os.path.isdir(dir): # skip non-existing directories
                fullpath = os.path.join(dir,program)
                if program_exists(fullpath):
                    return True
        # else, not found:
        if write_message:
            print 'program %s not found' % programs
        return False

    elif isinstance(programs, (list,tuple)):
        # initialize with None:
        for program in programs:  fullpaths[program] = None
        for program in programs:
            for dir in paths:
                if os.path.isdir(dir): # skip non-existing directories
                    fullpath = os.path.join(dir,program)
                    if program_exists(fullpath):
                        fullpaths[program] = fullpath
                        break  # stop when the program is found

    elif isinstance(programs, dict):
        # initialize with None:
        for program in programs.keys():  fullpaths[program] = None
        for program in programs.keys():
            for dir in paths:
                if os.path.isdir(dir): # skip non-existing directories
                    fullpath = os.path.join(dir,program)
                    if program_exists(fullpath):
                        fullpaths[program] = fullpath
                        break

    if write_message:
        missing = False
        for program in fullpaths.keys():
            if not fullpaths[program]:
                if isinstance(program, dict):
                    print "program '%s' (%s) not found" % \
                          (program,programs[program])
                else:  # list or tuple
                    print 'program "%s" not found' % program
                missing = True
        if missing:
            return None

    return fullpaths


def pathsearch(programs=[], modules=[], where=0):
    """
    Given a list of programs (programs) and modules (modules),
    search for these programs and modules in the directories
    in the PATH and PYTHONPATH environment variables, respectively.
    Check that each directory has read and write access too.
    The function is useful for checking that PATH and PYTHONPATH
    are appropriately set in CGI scripts.
    """

    path = os.environ['PATH']  # /usr/bin:/usr/local/bin:/usr/X11/bin
    paths = re.split(os.pathsep, path)
    for program in programs:
        found = 0
        for dir in paths:
            if os.path.isdir(dir): # skip non-existing directories
                # check read and execute access in this directory:
                if not os.access(dir, os.R_OK | os.X_OK):
                    print dir, 'does not have read/execute access'
                    sys.exit(1)
                fullpath = os.path.join(dir,program)
                if os.path.isfile(fullpath):
                    found = 1
                    if where:
                        print program, 'found in', dir
                    break
        if not found:
            print "The program '%s' was not found" % program
            print 'PATH contains the directories\n','\n'.join(paths)

    path = os.environ['PYTHONPATH']
    paths = re.split(os.pathsep, path)
    for module in modules:
        found = 0
        for dir in paths:
            if os.path.isdir(dir): # skip non-existing directories
                # check read and execute access in this directory:
                if not os.access(dir, os.R_OK | os.X_OK):
                    print dir, 'does not have read/execute access'
                    sys.exit(1)
                fullpath = os.path.join(dir,module) + '.py'
                if os.path.isfile(fullpath):
                    found = 1
                    if where:
                        print module, 'found in', dir
                    break
        if not found:
            print "The module '%s' was not found" % module
            print 'PYTHONPATH contains the directories\n',\
            '\n'.join(paths)

def preprocess_all_files(rootdir, options=''):
    """
    Run preprocess on all files of the form basename.p.ext
    in the directory with root rootdir. The output of each
    preprocess run is directed to basename.ext.

    @param rootdir: root of directory tree to be processed.
    @param options: options (string) to preprocess program.
    @return: nested list of ((dir, basename.p.ext, basename.p), success))
    tuples. success is boolean and indicates if the preprocess command
    was a success (or not).
    """
    # first check that the user has the preprocess script:
    if not findprograms('preprocess'):
        raise SystemError('The preprocess program could not be found')

    def treat_a_dir(fileinfo, d, files):
        warning = """\
#############################################################################
# WARNING: This is an autogenerated file!! Do not edit this file!!
# Edit the original file %s (on which preprocess.py will be run)
#############################################################################
"""

        for f in files:
            path = os.path.join(d, f)
            if '.p.' in f and not '.svn' in f:
                basename_dotp, ext = os.path.splitext(f)
                basename, dotp = os.path.splitext(basename_dotp)
                outfilename = basename + ext
                outpath = os.path.join(d, outfilename)
                cmd = 'preprocess %s %s > %s' % (options, path, outpath)
                #print cmd
                failure, output = system(cmd, failure_handling='warning')
                fileinfo.append( ((d, f, outfilename), not failure))
                # add warning header:
                _warning = warning % f
                _f = open(outpath, 'r'); _str = _f.read(); _f.close()
                _lines = _str.split('\n')
                if _lines[0].startswith('#!'):
                    _lines.insert(1, _warning)
                else:
                    _lines.insert(0, _warning)
                _str = '\n'.join(_lines)
                _f = open(outpath, 'w'); _f.write(_str); _f.close()

    info = []
    os.path.walk(rootdir, treat_a_dir, info)
    return info


def pow_eff(a,b, powfunc=math.pow):
    """
    Returns a^b. Smart function that happened to be slower
    than a straight math.pow.
    """
    if b == 2:
        return a*a
    elif b == 3:
        return a*a
    elif b == 4:
        h = a*a
        return h*h
    elif b == 1:
        return a
    elif abs(b) < 1.0E-15:  # x^0 ?
        return 1.0
    elif a == 0.0:
        return 0.0
    else:
        # check if b is integer:
        bi = int(b)
        if bi == b:
            r = 1
            for i in range(bi):
                r *= a
            return r
        else:
            if a < 0:
                raise ValueError('pow(a,b) with a=%g<0')
            else:
                return powfunc(a, b)


def lines2paragraphs(lines):
    """
    Return a list of paragraphs from a list of lines
    (normally holding the lines in a file).
    """
    p = []             # list of paragraphs to be returned
    firstline = 0      # first line in a paragraph
    currentline = 0    # current line in the file
    lines.insert(len(lines), '\n') # needed to get the last paragraph
    for line in lines:
        # for each new blank line, join lines from firstline
        # to currentline to a string defining a new paragraph:
        #if re.search(r'^\s*$', line):  # blank line?
        if line.isspace():  # blank line?
            if currentline > firstline:
                p.append(''.join(lines[firstline:currentline+1]))
                #print 'paragraph from line',firstline,'to',currentline
            # new paragraph starts from the next line:
            firstline = currentline+1
        currentline += 1
    return p


def oneline(infile, outfile):
    """
    Transform all paragraphs in infile (filename) to one-line strings
    and write the result to outfile (filename).
    """
    fi = open(infile, 'r')
    pp = lines2paragraphs(fi.readlines())
    fo = open(outfile, 'w')
    for p in pp:
        line = ' '.join(p.split('\n')) + '\n\n'
        fo.write(line)
    fi.close()
    fo.close()


def wrap(infile, outfile, linewidth=70):
    """
    Read infile (filename) and format the text such that each line is
    not longer than linewidth. Write result to outfile (filename).
    """
    fi = open(infile, 'r')
    fo = open(outfile, 'w')
    # the use of textwrap must be done paragraph by paragraph:
    from textwrap import wrap
    pp = lines2paragraphs(fi.readlines())
    for p in pp:
        #print 'paragraph:\n  "%s"' % p
        lines = wrap(p, linewidth)
        #print 'lines:\n', lines
        for line in lines:
            fo.write(line + '\n')
        fo.write('\n')
    fi.close()
    fo.close()


def fontscheme1(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('Helvetica', 13, 'normal')
    pulldown_font = ('Helvetica', 13, 'italic bold')
    scale_font    = ('Helvetica', 13, 'normal')
    root.option_add('*Font', default_font)
    root.option_add('*Menu*Font', pulldown_font)
    root.option_add('*Menubutton*Font', pulldown_font)
    root.option_add('*Scale.*Font', scale_font)

def fontscheme2(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('Helvetica', 10, 'normal')
    pulldown_font = ('Helvetica', 10, 'italic bold')
    scale_font    = ('Helvetica', 10, 'normal')
    root.option_add('*Font', default_font)
    root.option_add('*Menu*Font', pulldown_font)
    root.option_add('*Menubutton*Font', pulldown_font)
    root.option_add('*Scale.*Font', scale_font)

def fontscheme3(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('Fixed', 12, 'normal')
    root.option_add('*Font', default_font)

def fontscheme4(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('Fixed', 14, 'normal')
    root.option_add('*Font', default_font)

def fontscheme5(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('comic sans ms', 12, 'normal')
    root.option_add('*Font', default_font)

def fontscheme6(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('trebuchet ms', 12, 'normal bold')
    root.option_add('*Font', default_font)

def fontscheme7(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('verdana', 12, 'normal bold')
    root.option_add('*Font', default_font)

def fontscheme8(root):
    """Alternative font scheme for Tkinter-based widgets."""
    default_font  = ('verdana', 14, 'normal')
    root.option_add('*Font', default_font)




def movefiles(files, destdir, confirm=True, verbose=True, copy=True):
    """
    Move a set of files to a a destination directory tree,
    but let the original complete path be reflected in the
    destination tree.

    files         list of filenames
    destdir       root of destination directory tree
    confirm       let the user confirm movement of each file
    verbose       write out the original and new path of each file
    copy          True: copy, False: move

    The function is useful for backing up or temporarily moving
    files; the files are easily restored in their original
    locations since the original complete path is maintained in
    the destination directory tree.
    """
    if not os.path.isdir(destdir):
        os.mkdir(destdir)
    for file in files:
        perform_action = 'y'
        if confirm:
            print 'move %s to %s?' % (file, destdir)
            perform_action = sys.stdin.readline().strip()
        if perform_action in ('y', 'Y', 'yes', 'YES'):
            fullpath = os.path.abspath(file)
            # remove initial / (Unix) or C:\ (Windows):
            if sys.platform.startswith('win'):
                fullpath = fullpath[3:]
            else:  # Unix
                fullpath = fullpath[1:]
            newpath = os.path.join(destdir, fullpath)
            newdir = os.path.dirname(newpath)
            if not os.path.isdir(newdir): os.makedirs(newdir)
            shutil.copy2(file, newpath)
            if os.path.isfile(newpath):
                print 'fount',newpath
            s = 'copied'
            if not copy:  # pure move
                os.remove(file); s = 'moved'
            if verbose:
                print s, file, 'to', newpath

# backward compatibility:
from debug import debugregex, dump



class BackgroundCommand(threading.Thread):
    """
    Run a function call with assignment in the background.
    Useful for putting time-consuming calculations/graphics
    in the background in an interactive Python shell.

    >>> b=BG('f', g.gridloop, 'sin(x*y)-exp(-x*y)')
    >>> b.start()
    running gridloop('sin(x*y)-exp(-x*y)',) in a thread
    >>> # continue with other interactive tasks
    >>> b.finished
    True
    >>> b.f  # result of function call
    """

    def __init__(self, result='result', func=None, args=[], kwargs={}):
        self.result = result
        self.__dict__[self.result] = None
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.finished = False
        threading.Thread.__init__(self)
    def run(self):
        kw = [key+'='+self.kwargs[key] for key in self.kwargs]
        cmd = '%s=%s(%s,%s)' % (self.result, self.func.__name__,
                                ','.join(self.args), ','.join(kw))
        print 'running %s in a thread' % cmd
        self.__dict__[self.result] = self.func(*self.args,**self.kwargs)
        self.finished = True
        print cmd, 'finished'

BG = BackgroundCommand  # short form

class Download(threading.Thread):
    def __init__(self, url, filename):
        self.url = url;  self.filename = filename
        threading.Thread.__init__(self)
    def run(self):
        print 'Fetching', self.url
        urllib.urlretrieve(self.url, self.filename)
        print self.filename, 'is downloaded'

def hardware_info():
    """
    Return a dictionary of various types of hardware info from
    various modules.

    Recommended use::

      from scitools.misc import hardware_info
      import pprint; pprint.pprint(hardware_info())

    """
    result = {}

    # read cpuinfo file on Linux machines
    # (numpy.distutils.cpuinfo already does this)
    """
    infofile = '/proc/cpuinfo'
    cpuinfo = {}
    if os.path.isfile(infofile):
        f = open(infofile, 'r')
        for line in f:
            try:
                name, value = [w.strip() for w in line.split(':')]
            except:
                continue
            if name not in ('model', 'processor', 'stepping', 'flags',):
                cpuinfo[name] = value
            # Override a few with better names
            if name == 'model name':
                cpuinfo['CPU type'] = value
            elif name == 'cache size':
                cpuinfo['cache size'] = value
            elif name == 'cpu MHz':
                cpuinfo['CPU speed'] = value + ' Hz'
            elif name == 'vendor_id':
                cpuinfo['vendor ID'] = value
        f.close()
    result['file: /proc/cpuinfo'] = cpuinfo
    """
    import numpy.distutils.cpuinfo, copy
    info = copy.deepcopy(numpy.distutils.cpuinfo.cpu.info)
    # Delete some of the items
    for name in ('model', 'processor', 'stepping', 'flags',):
        if name in info:
            del info[name]
    result['numpy.distutils.cpuinfo.cpu.info'] = info

    # check out platform module from basic Python:
    import platform
    platform_info = {
        'uname': platform.uname(),
        'python version': platform.python_version(),
        'python build': platform.python_build(),
        'identifier': platform.platform()}
    result['platform module'] = platform_info

    # Trent Mick's platinfo module
    try:
        import platinfo2  # code.google.com/p/platinfo
        try:
            pi = platinfo.PlatInfo()
            result['platinfo2 module'] = pi.as_dict()
        except:
            pass
    except:
        pass

    return result


def memusage(_proc_pid_stat = '/proc/%s/stat'%(os.getpid())):
    """
    Return virtual memory size in bytes of the running python.
    Copied from the SciPy package (scipy_test.testing.py).
    """
    try:
        f=open(_proc_pid_stat,'r')
        l = f.readline().split(' ')
        f.close()
        return int(l[22])
    except:
        return


def _test_memusage(narrays=100, m=1000):
    """
    Test the memusage function for monitoring the memory usage.
    Generate narrays arrays of size (m,m). Keep the array
    with probability 0.5, otherwise delete a previously
    allocated array.
    """
    import random, Numeric
    random.seed(12)
    refs = []
    for i in range(narrays):
        a = Numeric.zeros((m,m), Numeric.Float)
        if random.random() > 0.5:
            refs.append(a)
        elif len(refs) > 0:
            del refs[0]
        mu = memusage()/1000000.0
        print 'memory usage: %.2fMb' % mu


def isiterable(data):
    """Returns true of data is iterable, else False."""
    try:
        iter(data)
    except TypeError:
        return False
    return True

def flatten(nested_data):
    """
    Return a flattened iterator over nested_data.

    >>> nested_list = [[1,2],3,[4,5,6,[7,[8,9]]]]
    >>> flat = [e for e in flatten(nested_list)]
    >>> flat
    [1, 2, 3, 4, 5, 6, 7, 8, 9]

    (Minor adjustment of code by Goncalo Rodrigues, see
    http://aspn.activestate.com/ASPN/Mail/Message/python-tutor/2302348)
    """
    it = iter(nested_data)
    for e in it:
        # note: strings are bad because, when iterated they return
        # strings, leading to an infinite loop
        if isiterable(e) and not isinstance(e, basestring):
            # recurse into iterators
            for f in flatten(e):
                yield f
        else:
            yield e

def primes(n):
    """
    Return the prime numbers <= n.
    Standard optimized sieve algorithm.
    """
    if n < 2:  return [1]
    if n == 2: return [1, 2]
    # do only odd numbers starting at 3
    s = range(3, n+1, 2)
    mroot = n**0.5
    half = len(s)
    i = 0
    m = 3
    while m <= mroot:
        if s[i]:
            j = (m*m-3)//2  # int div
            s[j] = 0
            while j < half:
                s[j] = 0
                j += m
        i = i+1
        m = 2*i+3
    return [1, 2] + [x for x in s if x]


def cmldict(argv, cmlargs=None, validity=0):
    """
    The cmldict function takes a dictionary cmlargs with default
    values for the command-line options and returns a modified form of
    this dictionary after the options given in the list argv are
    parsed and inserted. One will typically supply sys.argv[1:] as the
    argv argument. In case cmlargs is None, the dictionary is built
    from scratch inside the function.  The flag validity is false (0)
    if any option in argv can be inserted in cmlargs, otherwise the
    function will issue an error message if an option is not already
    present in cmlargs with a default value (notice that cmlargs=None
    and validity=1 is an incompatible setting).

    Example:
    cmlargs = {'p' : 0, 'file' : None, 'q' : 0, 'v' : 0}
    argv = "-p 2 --file out -q 0".split()
    p = cmldict(argv, cmlargs)

    p equals {'p' : 2, 'file' : out, 'q' : 0}
    """

    if not cmlargs:
        cmlargs = {}

    arg_counter = 0
    while arg_counter < len(argv):
        option = argv[arg_counter]
        if option[0] == '-':  option = option[1:]  # remove 1st hyphen
        else:
            # not an option, proceed with next sys.argv entry
            arg_counter += 1; continue
        if option[0] == '-':  option = option[1:]  # remove 2nd hyphen

        if not validity or option in cmlargs:
            # next argv entry is the value:
            arg_counter += 1
            value = argv[arg_counter]
            cmlargs[option] = value
        elif validity:
            raise ValueError("The option %s is not registered" % option)
        arg_counter += 1
    return cmlargs

def _cmldict_demo():
    args = "--m 9.1 --b 7 --c 0.1 -A 3.3".split()
    defaults = { 'm' : '1.8', 'func' : 'siny' }
    p = cmldict(args, defaults, 0)
    print p

    # shuffle values into other variables:
    m = p['m']
    b = p['b']
    # and so on (should have validity=1 to ensure that the
    # option keys are defined)

    # take action:
    for option in p.keys():
        if option == "m":
            print "option is m", p[option]
        elif option == "b":
            print "option is b", p[option]
        elif option == "c":
            print "option is c", p[option]
        elif option == "A":
            print "option is A", p[option]
        elif option == "func":
            print "option is func", p[option]

    args.append('--error'); args.append('yes')
    print "\nNow comes an exception (!)"
    p = cmldict(args, defaults, 1)

# used in StringFunction doc as an example:
def _test_function(x, c=0, a=1, b=2):
    if x > c:
        return a*(x-c) + b
    else:
        return -a*(x-c) + b

# -- tests ---
def f(a, b, max=1.2, min=2.2):  # some function
    print 'a=%g, b=%g, max=%g, min=%g' % (a,b,max,min)


class DoNothing(object):
    """
    Handy class for making other objects inactive.
    (DoNothing is a generic dispatcher, accepting anyting and
    doing nothing.)

    Whatever we do, we always get a DoNothing object, with which
    we can do whatever we want to, but nothing will happen.

    For example, say a plot function returns a plot object that
    is used widely in a code to create windows with visualizations
    on the screen, and you want to turn off all these visualizations:

    >>> from scitools.misc import DoNothing
    >>> plot = DoNothing('Plotting turned off')
    >>> viz = plot(u, wireframe=True, title='My plot')
    >>> type(viz)
    <class 'scitools.misc.DoNothing'>
    >>> viz.update(T)
    trying update but no action (DoNothing object)
    >>> q = viz.properties()
    trying properties but no action (DoNothing object)
    >>> type(q)
    <class 'scitools.misc.DoNothing'>

    One can turn the messages 'trying ... but no action' off by
    giving the argument silent=True to the constructor:

    >>> plot = DoNothing('Plotting turned off', silent=True)
    >>> viz = plot(u, wireframe=True, title='My plot')
    >>> viz.update(T)
    >>> q = viz.properties()
    """
    def __init__(self, *args, **kwargs):
        self.silent = kwargs.get('silent', False)

    def __call__(self, *args, **kwargs):
        return DoNothing()

    def __repr__(self):
        return ''

    def __str__(self):
        return repr(self)

    def __getattribute__(self, name):
        if name != 'silent' and not self.silent:
            print 'ignoring action "%s" (DoNothing object)' % name
        return DoNothing()

    def __iter__(self):
        return self

    def next(self):
        raise StopIteration()


class Recorder:
    """
    This class is a wrapper of a module or instance which will
    record all actions done with the module/instance.

    >>> from scitools.misc import Recorder
    >>> from scitools.std import plt, plot, linspace
    >>> x = linspace(-1,1,10)
    >>> y1 = x**2
    >>> y2 = x**3
    >>> plt._g = Recorder(plt._g)  # make the plot object record itself
    >>> plot(x, y1, 'r-',
    ...      x, y2, 'b-',
    ...      title='A test')
    [<scitools.easyviz.common.Line object at 0x1749c50>, <scitools.easyviz.common.Line object at 0x1749bd0>]
    >>> # look at what we have done with the plt._g object
    >>> plt._g.replay()
    reset()
    __call__('unset multiplot',)
    __call__('set datafile missing "nan"',)
    __call__('set key right top',)
    __call__('set title "A test"',)
    __call__('unset logscale x',)
    __call__('unset logscale y',)
    __call__('set autoscale',)
    __call__('set xrange[*:*]',)
    __call__('set yrange[*:*]',)
    __call__('set zrange[*:*]',)
    __call__('set size noratio',)
    __call__('set size nosquare',)
    __call__('set yrange [] noreverse',)
    __call__('set hidden3d',)
    __call__('unset colorbox',)
    __call__('set cbrange [*:*]',)
    __call__('set palette model RGB defined (0 "blue", 3 "cyan", 4 "green", 5 "yellow", 8 "red", 10 "black")',)
    __call__('unset view',)
    __call__('set view map',)
    __call__('set xtics',)
    __call__('set ytics',)
    __call__('set ztics',)
    __call__('set border 1+2+4+8+16 linetype -1 linewidth .4',)
    __call__('unset xlabel',)
    __call__('unset ylabel',)
    __call__('unset zlabel',)
    __call__('set border 4095 linetype -1 linewidth .4',)
    __call__('unset grid',)
    plot(<Gnuplot.PlotItems._FIFOFileItem instance at 0x174d998>,)
    replot(<Gnuplot.PlotItems._FIFOFileItem instance at 0x174db90>,)

    """
    def __init__(self, obj):
        self.obj = obj
        self.recorder = []

    def __getattr__(self, name):
        return _RecordHelper(self.obj, name, self.recorder)

    def replay(self):
        for name, args, kwargs in self.recorder:
            s = name + '('
            if args:
                s += str(args)[1:-1]
            if kwargs:
                s += ', ' + ', '.join(['%s=%s' % (key, kwargs[key]) for key in kwargs])
            s += ')'
            print s

class _RecordHelper:
    def __init__(self, obj, name, recorder):
        self.obj, self.name, self.recorder = obj, name, recorder

    def __call__(self, *args, **kwargs):
        self.recorder.append((self.name, args, kwargs))
        if hasattr(self.obj, self.name):
            return getattr(self.obj, self.name)(*args, **kwargs)
        else:
            raise AttributeError('%s has no attribute %s', (self.obj, name))



def fix_latex_command_regex(pattern, application='match'):
    """
    Given a pattern for a regular expression match or substitution,
    the function checks for problematic patterns commonly
    encountered when working with LaTeX texts, namely commands
    starting with a backslash.

    For a pattern to be matched or substituted, and extra backslash is
    always needed (either a special regex construction like \w leads
    to wrong match, or \c leads to wrong substitution since \ just
    escapes c so only the c is replaced, leaving an undesired
    backslash). For the replacement pattern in a substitutions, specified
    by the application='replacement' argument, a backslash
    before any of the characters abfgnrtv must be preceeded by an
    additional backslash.

    The application variable equals 'match' if pattern is used for
    a match and 'replacement' if pattern defines a replacement
    regex in a re.sub command.

    Caveats: let pattern just contain LaTeX commands, not combination
    of commands and other regular expressions (\s, \d, etc.) as the
    latter will end up with an extra undesired backslash.

    Here are examples on failures:

    >>> re.sub(r'\begin\{equation\}', r'\[', r'\begin{equation}')
    '\\begin{equation}'
    >>> # match of mbox, not \mbox, and wrong output:
    >>> re.sub(r'\mbox\{(.+?)\}', r'\fbox{\g<1>}', r'\mbox{not}')
    '\\\x0cbox{not}'

    Here are examples on using this function:

    >>> from scitools.misc import fix_latex_command_regex as fix
    >>> pattern = fix(r'\begin\{equation\}', application='match')
    >>> re.sub(pattern, r'\[', r'\begin{equation}')
    '\\['
    >>> pattern = fix(r'\mbox\{(.+?)\}', application='match')
    >>> replacement = fix(r'\fbox{\g<1>}', application='replacement')
    >>> re.sub(pattern, replacement, r'\mbox{not}')
    '\\fbox{not}'

    Avoid mixing LaTeX commands and ordinary regular expression
    commands, e.g.:

    >>> pattern = fix(r'\mbox\{(\d+)\}', application='match')
    >>> pattern
    '\\\\mbox\\{(\\\\d+)\\}'
    >>> re.sub(pattern, replacement, r'\mbox{987}')
    '\\mbox{987}'  # no substitution, no match
    """
    import string
    problematic_letters = string.ascii_letters if application == 'match' \
                          else 'abfgnrtv'

    for letter in problematic_letters:
        problematic_pattern = '\\' + letter

        if letter == 'g' and application == 'replacement':
            # no extra \ for \g<...> in pattern
            if r'\g<' in pattern:
                continue

        ok_pattern = '\\\\' + letter
        if problematic_pattern in pattern and not ok_pattern in pattern:
            pattern = pattern.replace(problematic_pattern, ok_pattern)
    return pattern



if __name__ == '__main__':
    try:
        task = sys.argv[1]
    except:
        task = ''

    if task == 'debugregex':
        r = r'<(.*?)>'
        s = '<r1>is a tag</r1> and <s1>s1</s1> is too.'
        print debugregex(r,s)
        print debugregex(r'(\d+\.\d*)','a= 51.243 and b =1.45')




#!/usr/bin/env python
'''
Module for estimating convergence rate of numerical algorithms,
based on data from a set of experiments.

It is recommended to vary only
one discretization parameter h. In spatial problems, h is the mesh
spacing dx, dy, or dz.
In time-dependent problems, where the expected convergence rate is
h^p + dt^q, choose h^p/dt^q = const.
Use class ``OneDiscretizationPrm``, or the function ``convergence_rate``,
or the ``analyze_filedata`` convenience function.

Here is a simple example::

    h = [0.2, 0.1, 0.05, 0.025]  # discretization parameters
    e = [compute_error(hi) for hi in h]

    rates, C = OneDiscretizationPrm.pairwise_rates(h, e)
    print C, rates  # see if rates converges
    print 'final rate:', rates[-1]

There is support for fitting more general error models, like
C1*h^r1 + C2*h*dt^r2, with nonlinear least squares, but in that case it
is more challenging to find sound fits.
'''

from scitools.misc import import_module
from numpy import zeros, array, asarray, log10, transpose, linalg, linspace
import sys
from scitools.std import plot

__all__ = ['ManyDiscretizationPrm',
           'OneDiscretizationPrm',
           ]

log = log10
inv_log = lambda x: 10**x

# The classes in this module have only static methods, i.e.,
# classes are merely name spaces for related functions.


class OneDiscretizationPrm(object):
    """
    Tools for fitting an error model with only one discretization
    parameter: e = C*h^2.
    """

    def error_model(p, d):
        """Return e = C*h**a, where p=[C, a] and h=d[0]."""
        C, a = p
        h = d[0]
        return C*h**a

    def loglog_error_model(p, d):
        """As ``error_model``, but log-log data was used in the estimation."""
        C, a = p
        h = d[0]
        return log(C) + a*log(h)


    def linear_loglog_fit(d, e):
        """
        Linear least squares algorithm.
        Suitable for problems with only one distinct
        discretization parameter.
        `d` is the sequence of discretization parameter values, and
        `e` is the sequence of corresponding error values.

        The error model the data is supposed to fit reads
        ``log(e[i]) = log(C[i]) + a*log(d[i])``.
        """
        A = transpose(array([d, zeros(len(d))+1]))
        sol = linalg.lstsq(A, e)
        a, logC = sol[0]
        C = inv_log(logC)
        return a, C

    def nonlinear_fit(d, e, p0):
        """
        ======== ===========================================================
        Name     Description
        ======== ===========================================================
        d        list of values of the (single) discretization
                 parameter in each experiment:
                 ``d[i]`` provides the values of the discretization,
                 parameter in experiement no. i.
        e        list of error values; ``e = (e_1, e_2, ...)``,
                 ``e[i]`` is the error associated with the parameters
                 ``d[i]``
        p0       starting values for the unknown parameters vector
        return   r, C; r is the exponent, C is the factor in front
        ======== ===========================================================
        """
        if len(d) != len(e):
            raise ValueError('d and e must have the same length')
        if not isinstance(d[0], (float,int)):
            raise TypeError('d must be an array of numbers, not %s' % \
                            str(type(d[0])))
        # transform d and e to the data format required by
        # the Scientific package:
        data = []
        for d_i, e_i in zip(d, e):
            data.append(((d_i,) , e_i))  # recall (a,) conversion to tuple
        leastSquaresFit = import_module('Scientific.Functions.LeastSquares',
                                        'leastSquaresFit')
        sol = leastSquaresFit(OneDiscretizationPrm.error_model, p0, data)
        C = sol[0][0]
        a = sol[0][1]
        return a, C


    def pairwise_rates(d, e):
        """
        Compare convergence rates, where each rate is based on
        a formula for two successive experiments.
        """
        if len(d) != len(e):
            raise ValueError('d and e must have the same length')

        rates = []
        for i in range(1, len(d)):
            try:
                rates.append( log(e[i-1]/e[i])/log(float(d[i-1])/d[i]) )
            except (ZeroDivisionError, OverflowError):
                rates.append(0)
        # estimate C from the last data point:
        try:
            C = e[-1]/d[-1]**rates[-1]
        except:
            C = 0
        return rates, C


    def analyze(d, e, initial_guess=None,
                plot_title='', filename='tmp.ps'):
        """
        Run linear, nonlinear and successive rates models.
        Plot results for comparison of the three approaches.

        ============== =================================================
        Argument       Description
        ============== =================================================
        d              list/array of discretization parameter
        e              errors corresponding to d
        initial_guess  estimate of convergence rate parameters
        plot_title     title in plot
        filename       name of plot file
        ============== =================================================
        """
        # convert to NumPy arrays:
        d = asarray(d, float);  e = asarray(e, float)

        # linear least squares fit:
        a1, C1 = OneDiscretizationPrm.linear_loglog_fit(log(d), log(e))
        print "linear LS fit: const=%e rate=%.1f" % (C1, a1)

        # nonlinear least squares fit (no log-log):
        a2, C2 = OneDiscretizationPrm.nonlinear_fit(d, e, initial_guess)
        print "nonlinear LS fit: const=%e rate=%.1f" % (C2, a2)

        # pairwise estimate of the rate:
        rates, C3 = OneDiscretizationPrm.pairwise_rates(d, e)
        a3 = rates[-1]
        print "pairwise fit: const=%e rate=%.1f" % (C3, a3)
        print "all rates:", rates

        if C1 < 0 or C2 < 0 or C3 < 0:
            raise ValueError('Some fits give negative const value! Cannot plot.')
            return

        # else log plot:
        log_d1 = linspace(log(d[0]), log(d[-1]), 2)
        log_e1 = log(C1) + a1*log_d1
        log_e2 = log(C2) + a2*log_d1
        log_e3 = log(C3) + a3*log_d1
        plot(log(d), log(e), 'yo',
             log_d1, log_e1, 'r-',
             log_d1, log_e2, 'b-',
             log_d1, log_e3, 'g-',
             legend=('data',
                     'linear LS log-log fit: %.1f*h^%.1f' % (log(C1), a1),
                     'nonlinear LS fit: %.1f*h^%.1f' % (log(C2), a2),
                     'successive rates, last two experiments: %.1f*h^%.1f' % (log(C3), a3)),
             #axis=[log_d1[-1], log_d1[0], 1.3*log(e[-1]), 1.5*log(e[0])],
             hardcopy=filename)

    analyze = staticmethod(analyze)

    error_model = staticmethod(error_model)
    loglog_error_model = staticmethod(loglog_error_model)
    linear_loglog_fit = staticmethod(linear_loglog_fit)
    nonlinear_fit = staticmethod(nonlinear_fit)
    pairwise_rates = staticmethod(pairwise_rates)

# convenience function:
def convergence_rate(discretization_prm, error):
    """
    Given two lists/arrays with discretization parameters and
    corresponding errors in a numerical method (element no. i
    in the two lists must correspond to each other), this
    function assumes an error formula of the form E=C*d^r,
    where E is the error and d is the discretization parameter.
    The function returns C and r.

    Method used: OneDiscretizationPrm.pairwise_rates is called
    and the final r value is used for return. A check that
    the rates are converging (the last three) is done.
    """
    rates, C = OneDiscretizationPrm.pairwise_rates(discretization_prm, error)
    # check that there is no divergence at the end of
    # the series of experiments
    differences = [rates[i] - rates[i-1] for i in range(len(rates)-1)]
    # the differences between the rates should decrease, at least
    # toward the end
    try:
        if differences[-1] > differences[-2]:
            raise ValueError('The pairwise convergence rates do not '\
                             'decrease toward the end:\n%s' % \
                             str(rates))
    except IndexError:
        pass  # not enough data to check the differences list
    return C, rates[-1]

class ManyDiscretizationPrm(object):
    """
    General tool for fitting an error model containing an
    arbitrary number of discretization parameters.
    The error is a weighted sum of each discretization parameter
    raised to a real expoenent. The weights and exponents are
    the unknown parameters to be fitted by a nonlinear
    least squares procedure.
    """

    def error_model(p, d):
        """
        Evaluate the theoretical error model (sum of C*h^r terms):
        sum_i p[2*i]*d[i]**p[2*i+1]

        ====== =======================================================
        Name   Description
        ====== =======================================================
        p      sequence ofvalues of  parameters (estimated)
        d      sequence of values of (known) discretization parameters
        return error evaluated
        ====== =======================================================

        Note that ``len(p)`` must be ``2*len(d)`` in this model since
        there are two parameters (constant and exponent) for each
        discretization parameter.
        """
        if len(p) != 2*len(d):
            raise ValueError('len(p)=%d != 2*len(d)=%d' % (len(p),2*len(d)))
        sum = 0
        for i in range(len(d)):
            sum += p[2*i] * d[i]**p[2*i+1]
        return sum

    def nonlinear_fit(d, e, initial_guess):
        """
        ============== ================================================
        Argument       Description
        ============== ================================================
        d              list of values of the set of discretization
                       parameters in each experiment:
                       ``d = ((d_1,d_2,d_3),(d_1,d_2,d_3,),...)``;
                       ``d[i]`` provides the values of the
                       discretization parameters in experiement no. i.
        e              list of error values; ``e = (e_1, e_2, ...)``:
                       ``e[i]`` is the error associated with the
                       parameters ``d[i]``
        initial_guess  the starting value for the unknown
                       parameters vector
        return         list of fitted parameters
        ============== ================================================
        """
        if len(d) != len(e):
            raise ValueError('len(d) != len(e)')
        # transform d and e to the data format required by
        # the Scientific package:
        data = []
        for d_i, e_i in zip(d, e):
            if isinstance(d_i, (float, int)):
                data.append(((d_i,), e_i))
            else:  # d_i is tuple, list, array, NumArray, ...
                data.append((d_i, e_i))
        leastSquaresFit = import_module('Scientific.Functions.LeastSquares',
                                        'leastSquaresFit')
        sol = leastSquaresFit(ManyDiscretizationPrm.error_model,
                              initial_guess, data)
        # return list of fitted parameters (p in error_model)
        # (sol[1] is a measure of the quality of the fit)
        return sol[0]

    error_model = staticmethod(error_model)
    nonlinear_fit = staticmethod(nonlinear_fit)


def _test1():
    """Single discretization parameter test."""
    import random
    random.seed(1234)
    n = 7
    h = 1
    e = [];  d = []
    for i in range(7):
        h /= 2.0
        error = OneDiscretizationPrm.error_model((4,2), (h,))
        error += random.gauss(0, 0.1*error)  # perturb data
        d.append(h)
        e.append(error)
    OneDiscretizationPrm.analyze(d, e, initial_guess=(3,2))

def analyze_filedata():
    """
    Reads ``f C r`` from the command line, where ``f`` is the name of
    a file, ``C`` and ``r`` are the initial guesses of the parameters
    in the error model E = C*h^r. A 4th command-line argument can be
    the plot title.
    """
    # read filename and initial guess of C and r in error formula E=C*h^r
    f = open(sys.argv[1], 'r')
    C = float(sys.argv[2])
    r = float(sys.argv[3])
    try:
        plot_title = sys.argv[4]
    except:
        plot_title = ''
    from scitools.filetable import read
    data = read(f)
    print data
    OneDiscretizationPrm.analyze(data[:,0], data[:,1],
                                 initial_guess=(C,r),
                                 plot_title=plot_title)

# extensions:
# example with dx, dy and dt
# same example, but with factors to get a common rate
# dx, dt tables and experiments with whole table, one
# column and one row, and the diagonal

if __name__ == '__main__':
    if len(sys.argv) == 1:
        print 'Usage: %s filename C-guess r-guess [plot-title]' % sys.argv[0]
    elif sys.argv[1] == 'example':
        _test1()
    else:
        analyze_filedata()



#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/visualize/f5-3371.html

from scitools.easyviz import *
from time import sleep
from scipy import io

setp(interactive=False)

mri = io.loadmat('mri_matlab_v6.mat')
D = mri['D']
D = squeeze(D)

image_num = 8

x = xlim;
y = ylim;

# Displaying a 2-D Contour Slice:
contourslice(D,[],[],image_num,indexing='xy')
axis('ij')
xlim(x)
ylim(y)
daspect([1,1,1])
colormap('default')
show()

#hardcopy('tmp_contourslice2a.eps')
#hardcopy('tmp_contourslice2a.png')

figure()
BUG = 1
phandles = contourslice(D,[],[],[1,12,19,27-BUG],8,indexing='xy')
view(3)
axis('tight')
#set(phandles,'LineWidth',2)
setp(phandles, linewidth=4)
show()
#sleep(3)
raw_input('Press Return key to quit: ')

## hardcopy('tmp_contourslice2b.eps')
## hardcopy('tmp_contourslice2b.png')



#!/usr/bin/env python
"""Test scitools profiler by calling two functions that take some time."""

import os, sys
N = 10000  # big number

def f1():
    for i in range(N):
        # do something
        os.path.isfile(sys.argv[0])

def f2():
    for i in range(N+10):
        # do something
        os.path.isdir(os.pardir)

for i in range(4):
    f1()
f2()




#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/visualize/f5-3371.html

from scitools.easyviz import *
from time import sleep
from scipy import io

setp(interactive=False)

# Displaying an Isosurface:
mri = io.loadmat('mri_matlab_v6.mat')
D = mri['D']
#Ds = smooth3(D);

isosurface(D,5,indexing='xy')
#hiso = isosurface(Ds,5),
#	'FaceColor',[1,.75,.65],...
#	'EdgeColor','none');
shading('interp')

# Adding an Isocap to Show a Cutaway Surface:
#hcap = patch(isocaps(D,5),...
#	'FaceColor','interp',...
#	'EdgeColor','none');
#colormap(map)

# Define the View:
view(45,30)
axis('tight')
daspect([1,1,.4])

# Add Lighting:
#lightangle(45,30);
#set(gcf,'Renderer','zbuffer'); lighting phong
#isonormals(Ds,hiso)
#set(hcap,'AmbientStrength',.6)
#set(hiso,'SpecularColorReflectance',0,'SpecularExponent',50)

show()

raw_input('Press Return key to quit: ')

#hardcopy('tmp_isosurf2a.eps')
#hardcopy('tmp_isosurf2a.png')



#!/usr/bin/env python
r"""

Example usage:
--------------

Redirect stderr using the 'with'-statement:

>>> from __future__ import with_statement
>>> from redirect_io import *
>>> with hidden_stderr():
...     print >> sys.stderr, 'Divide By Cucumber Error'
>>>

Redirect stderr to stout:

>>> with hidden_stderr(sys.stdout):
...     print >> sys.stderr, 'Where am I ?'
Where am I ?
>>>

Optinal usage without 'with'-statement:

>>> _redirect_err()
>>> try:
...     print >> sys.stderr, " +++ Divide by Cucumber Error, "\
...                          "Please ReInstall Universe And Reboot +++"
... finally:
...     _return_err()
>>>

Fetch previous error messages:

>>> print _tmp_err.getvalue() 
Divide By Cucumber Error
 +++ Divide by Cucumber Error, Please ReInstall Universe And Reboot +++
<BLANKLINE>
>>> 
"""
from __future__ import with_statement # Comment to run _test_with_statement
__author__ = 'Rolv Erlend Bredesen <rolv@simula.no>'
__all__ = ['sys', '_tmp_err', 'hidden_stderr', '_redirect_err', '_return_err']

import __future__
import StringIO 
import sys
from contextlib import contextmanager  

if float(sys.version[:3]) < 2.5:
    raise ImportError("This module requires version >= 2.5 of python")

_std_err = sys.stderr
_tmp_err =  StringIO.StringIO() # stream for hidden stderr
_tmp_err.__exit__ = _tmp_err.flush # Add __exit__ method

@contextmanager # 'with' requires __enter__ and __exit__
def hidden_stderr(stream=_tmp_err):
    sys.stderr = stream
    try:
        yield None
    finally:
        sys.stderr = _std_err

# For error stream with python version < 2.5
def _redirect_err():
    sys.stderr = _tmp_err
        
def _return_err():
    sys.stderr = _std_err
    
        
def _test_with_statement():
    """
    If the word 'with', which is a reserved keyword from python version 2.6,
    is used in python2.5 code then a warning will be printed to sys.stderr
    when the code is bytecompiled.

    In this example we look at capturing the warning messages
    """
    assert 'with_statement' not in globals()
    assert sys.version[:3] == '2.5'
    # Force Error
    statement = 'with=3'
    try:
        c = compile(statement, '/dev/null', 'exec',
                    __future__.with_statement.compiler_flag)
    except SyntaxError:
        print "Success: '%s' did not compile" %statement
    else: 
        raise Exception("Statement: '%s' should not have been compiled" \
                        % statement)
    
    # Setup error redirection 
    std_err = sys.stderr
    tmp_err = StringIO.StringIO()
    if float(sys.version[:3]) < 2.5:
        raise ImportError, "This module requires version 2.5 of python"
        
    # Compile with error redirection
    statement1 = 'with=3'
    statement2 = 'print "with>>", with'
    print '>>>', statement1
    print '>>>', statement2
    
    # Test compilation of code using the reserved keyword with
    # Redirection standard error under compilation to prevent warning
    _redirect_err()
    try:
        c = compile(statement1, '/dev/null', 'single')
    finally:
        _return_err()

    # No warning here
    exec c in {}, locals()

    _redirect_err()
    try:
        # This one will give a warning
        exec statement2     
    finally:
        _return_err()
    print "Here is the redirected error stream:\n",  tmp_err.getvalue()

    # 'with'-example
    print "Testing the use of the 'with'-statement"
    c = compile(
        r"""'''In this example we test the use of the 'with'-statement'''
        
#from __future__ import with_statement # using proper compile flag instead
from contextlib import contextmanager  # 'with' requires __enter__ and __exit__

err_ = StringIO.StringIO()

@contextmanager
def stderr_redirected(stream):
    err = sys.stderr
    sys.stderr = stream
    try:
        yield None
    finally:
        sys.stderr = err

statement = 'with_=3' # we have imported the with_statement

with stderr_redirected(err_):
    print >> sys.stderr, "Can you see me?"
    c_ = compile(statement, '/dev/null', 'single', 0, 0)


""", '/dev/null', 'exec', __future__.with_statement.compiler_flag)
    exec c

    print "\nIt appears the with_statement worked."
    print "Here is the redirected error stream:\n",  err_.getvalue()
    
    
if __name__ == '__main__':
    if 'with_statement' in globals(): 
        import doctest
        import unittest
        suite = unittest.TestSuite()
        suite.addTest(doctest.DocTestSuite())
        runner = unittest.TextTestRunner(sys.stdout, verbosity=1)
        runner.run(suite)
    else:
        # To run you must comment    
        # 'from __future__ import with_statement ' at start of file
        _test_with_statement()
        
